@@ -1,4 +1,4 @@
-/*  $Id: radius.c 6155 2003-01-19 19:58:25Z rra $
+/*  $Id: radius.c 9250 2011-07-17 18:06:00Z iulius $
 **
 **  Authenticate a user against a remote radius server.
 */
@@ -20,9 +20,9 @@
 #include "inn/innconf.h"
 #include "inn/md5.h"
 #include "inn/messages.h"
-#include "libinn.h"
-#include "nntp.h"
-#include "paths.h"
+#include "inn/libinn.h"
+#include "inn/nntp.h"
+#include "inn/paths.h"
 #include "conffile.h"
 
 #include "libauth.h"
@@ -36,7 +36,7 @@ typedef struct _auth_req {
     unsigned char	id;
     unsigned short	length;
     unsigned char	vector[AUTH_VECTOR_LEN];
-    unsigned char	data[NNTP_STRLEN*2];
+    unsigned char	data[NNTP_MAXLEN_COMMAND*2];
     int			datalen;
 } auth_req;
 
@@ -74,25 +74,25 @@ typedef struct _sending_t {
 #define RADsource  18
 
 static CONFTOKEN radtoks[] = {
-  { RADlbrace,   "{" },
-  { RADrbrace,   "}" },
-  { RADserver,   "server" },
-  { RADhost,     "radhost:" },
-  { RADsecret,   "secret:" },
-  { RADport,     "radport:" },
-  { RADlochost,  "lochost:" },
-  { RADlocport,  "locport:" },
-  { RADprefix,   "prefix:" },
-  { RADsuffix,   "suffix:" },
-  { RADsource,   "ignore-source:" },
-  { 0, 0 }
+    { RADlbrace,        (char *) "{"                    },
+    { RADrbrace,        (char *) "}"                    },
+    { RADserver,        (char *) "server"               },
+    { RADhost,          (char *) "radhost:"             },
+    { RADsecret,        (char *) "secret:"              },
+    { RADport,          (char *) "radport:"             },
+    { RADlochost,       (char *) "lochost:"             },
+    { RADlocport,       (char *) "locport:"             },
+    { RADprefix,        (char *) "prefix:"              },
+    { RADsuffix,        (char *) "suffix:"              },
+    { RADsource,        (char *) "ignore-source:"       },
+    { 0,                NULL                            }
 };
 
 static rad_config_t *get_radconf(void)
 {
   rad_config_t *new;
 
-  new = xmalloc(sizeof(rad_config_t));
+  new = xcalloc(1, sizeof(rad_config_t));
   new->next = NULL;
 
   return new;
@@ -104,7 +104,6 @@ static int read_config(char *authfile, rad_config_t *radconf)
     rad_config_t *radconfig=NULL;
     CONFFILE *file;
     CONFTOKEN *token;
-    char *server;
     int type;
     char *iter;
 
@@ -118,7 +117,6 @@ static int read_config(char *authfile, rad_config_t *radconf)
           die("expected server keyword on line %d", file->lineno);
 	if ((token = CONFgettoken(0, file)) == NULL)
           die("expected server name on line %d", file->lineno);
-	server = xstrdup(token->name);
 	if ((token = CONFgettoken(radtoks, file)) == NULL 
 	    || token->type != RADlbrace)
           die("expected { on line %d", file->lineno);
@@ -211,14 +209,14 @@ static int read_config(char *authfile, rad_config_t *radconf)
 #define RAD_NAS_IP_ADDRESS      4       /* IP address */
 #define RAD_NAS_PORT            5       /* Integer */
 
-static void req_copyto (auth_req to, sending_t *from)
+static void req_copyto (auth_req *to, sending_t *from)
 {
-    to = from->req;
+    *to = from->req;
 }
 
-static void req_copyfrom (sending_t *to, auth_req from)
+static void req_copyfrom (sending_t *to, auth_req *from)
 {
-    to->req = from;
+    to->req = *from;
 }
 
 static int rad_auth(rad_config_t *radconfig, char *uname, char *pass)
@@ -267,7 +265,7 @@ static int rad_auth(rad_config_t *radconfig, char *uname, char *pass)
 	sreq->next = new;
 	sreq = sreq->next;
       }
-      req_copyto(req, sreq);
+      req_copyto(&req, sreq);
   
       /* first, build the sockaddrs */
       memset(&sinl, '\0', sizeof(sinl));
@@ -392,7 +390,7 @@ static int rad_auth(rad_config_t *radconfig, char *uname, char *pass)
       sreq->reqlen = req.length;
       req.length = htons(req.length);
 
-      req_copyfrom(sreq, req);
+      req_copyfrom(sreq, &req);
 
       /* Go to the next record in the list */
       config = config->next;
@@ -415,7 +413,7 @@ static int rad_auth(rad_config_t *radconfig, char *uname, char *pass)
     for(done = 0; authtries > 0 && !done; authtries--) {
       for (config = radconfig, sreq = reqtop; sreq != NULL && !done;
 	   config = config->next, sreq = sreq->next){
-	req_copyto(req, sreq);
+	req_copyto(&req, sreq);
 
 	/* send out the packet and wait for reply. */
 	if (sendto(sock, (char *)&req, sreq->reqlen, 0, 
@@ -444,6 +442,8 @@ static int rad_auth(rad_config_t *radconfig, char *uname, char *pass)
 	    now = time(0);
 	    tmout.tv_sec = end - now + 1;
 	    tmout.tv_usec = 0;
+	    warn("timeout talking to remote radius server %s:%d",
+                 inet_ntoa(sreq->sinr.sin_addr), ntohs(sreq->sinr.sin_port));
 	    continue;
 	}
 	slen = sizeof(sinl);
@@ -479,13 +479,12 @@ static int rad_auth(rad_config_t *radconfig, char *uname, char *pass)
 	close(sock);
 	return (req.code == PW_AUTHENTICATION_ACK) ? 0 : -1;
 	done = 1;
-	req_copyfrom(sreq, req);
+	req_copyfrom(sreq, &req);
 	break;
       }
     }
     if (authtries == 0)
-        warn("cannot talk to remote radius server %s:%d",
-             inet_ntoa(sreq->sinr.sin_addr), ntohs(sreq->sinr.sin_port));
+        warn("cannot talk to any remote radius servers");
     return(-2);
 }
 
@@ -534,7 +533,7 @@ int main(int argc, char *argv[])
     if (argc != optind)
       exit(2);
     if (!havefile) {
-      radius_config = concatpath(innconf->pathetc, _PATH_RADIUS_CONFIG);
+      radius_config = concatpath(innconf->pathetc, INN_PATH_RADIUS_CONFIG);
       read_config(radius_config, &radconfig);
   
       free(radius_config);
@@ -559,6 +558,6 @@ int main(int argc, char *argv[])
             retval);
 
     /* radius password matches! */
-    printf("User:%s\n", authinfo->username);
+    print_user(authinfo->username);
     exit(0);
 }
