@@ -1,4 +1,4 @@
-/*  $Id: nntpget.c 8469 2009-05-16 12:43:53Z iulius $
+/*  $Id: nntpget.c 8921 2010-01-22 23:33:08Z iulius $
 **
 **  Connect to a remote site, and get news from it to offer to our local
 **  server.  Read list on stdin, or get it via NEWNEWS command.  Writes
@@ -22,7 +22,7 @@
 #include "inn/innconf.h"
 #include "inn/messages.h"
 #include "inn/libinn.h"
-#include "nntp.h"
+#include "inn/nntp.h"
 #include "inn/paths.h"
 
 /*
@@ -43,7 +43,7 @@ typedef struct _SITE {
 */
 static struct iovec	SITEvec[2];
 static char		SITEv1[] = "\r\n";
-static char		READER[] = "mode reader";
+static char		READER[] = "MODE READER";
 static unsigned long	STATgot;
 static unsigned long	STAToffered;
 static unsigned long	STATsent;
@@ -161,7 +161,7 @@ SITEquit(SITE *sp)
 {
     char	buff[NNTP_MAXLEN_COMMAND];
 
-    SITEwrite(sp, "quit", 4);
+    SITEwrite(sp, "QUIT", 4);
     SITEread(sp, buff);
 }
 
@@ -359,7 +359,7 @@ main(int ac, char *av[])
 	if (Offer) {
 	    /* See if the local server wants it. */
 	    STAToffered++;
-	    snprintf(buff, sizeof(buff), "ihave %s", mesgid);
+	    snprintf(buff, sizeof(buff), "IHAVE %s", mesgid);
 	    if (!SITEwrite(Local, buff, (int)strlen(buff))
 	     || !SITEread(Local, buff)) {
                 syswarn("cannot offer %s", mesgid);
@@ -370,7 +370,7 @@ main(int ac, char *av[])
 	}
 
 	/* Try to get the article. */
-	snprintf(buff, sizeof(buff), "article %s", mesgid);
+	snprintf(buff, sizeof(buff), "ARTICLE %s", mesgid);
 	if (!SITEwrite(Remote, buff, (int)strlen(buff))
 	 || !SITEread(Remote, buff)) {
             syswarn("cannot get %s", mesgid);
