@@ -1,4 +1,4 @@
-##  $Id: newgroup.pl 9001 2010-03-14 21:04:23Z iulius $
+##  $Id: newgroup.pl 9304 2011-08-04 22:10:47Z iulius $
 ##
 ##  newgroup control message handler.
 ##
@@ -133,13 +133,13 @@ sub control_newgroup {
             $status = 'be made moderated';
         } else {
             if ($ngdesc eq $olddesc) {
-                $status = 'no change';
+                $status = 'not change';
             } else {
                 $status = 'have a new description';
             }
         }
     } elsif (not $approved) {
-        $status = 'unapproved';
+        $status = 'be unapproved';
     } else {
         $status = 'be created';
     }
@@ -169,7 +169,7 @@ sub control_newgroup {
         return;
     }
 
-    if ($action eq 'mail' and $status !~ /(no change|unapproved)/) {
+    if ($action eq 'mail' and $status !~ /(not change|be unapproved)/) {
         my $mail = sendmail("newgroup $groupname $modcmd $sender");
         print $mail <<END;
 $sender asks for $groupname
@@ -196,8 +196,8 @@ END
             logmsg("skipping newgroup $groupname $modcmd $sender"
                 . " (would $status)");
         }
-    } elsif ($action eq 'doit' and $status ne 'unapproved') {
-        if ($status ne 'no change') {
+    } elsif ($action eq 'doit' and $status ne 'be unapproved') {
+        if ($status ne 'not change') {
             # The status 'be made (un)moderated' prevails over
             # 'have a new description' so it is executed.
             ctlinnd('newgroup', $groupname, $modcmd, $sender)
@@ -208,7 +208,7 @@ END
 
         if ($log) {
             logger($log, "newgroup $groupname $modcmd $status $sender",
-                   $article) if ($log ne 'mail' or $status ne 'no change');
+                   $article) if ($log ne 'mail' or $status ne 'not change');
         }
     }
     return;
@@ -216,7 +216,11 @@ END
 
 sub update_desc {
     my ($name, $desc) = @_;
-    shlock("$INN::Config::locks/LOCK.newsgroups");
+    my $lockfile = "$INN::Config::locks/LOCK.newsgroups";
+
+    # Acquire a lock.
+    INN::Utils::Shlock::lock($lockfile, 60) or logdie("Cannot create lockfile $lockfile");
+
     my $tempfile = "$INN::Config::newsgroups.$$";
     open(NEWSGROUPS, $INN::Config::newsgroups)
         or logdie("Cannot open $INN::Config::newsgroups: $!");
@@ -237,7 +241,10 @@ sub update_desc {
     close NEWSGROUPS;
     rename($tempfile, $INN::Config::newsgroups)
         or logdie("Cannot rename $tempfile: $!");
-    unlink("$INN::Config::locks/LOCK.newsgroups", $tempfile);
+    unlink($tempfile);
+
+    # Unlock.
+    INN::Utils::Shlock::unlock($lockfile);
 }
 
 # Check the group name.  This is partially derived from C News.
