@@ -1,4 +1,4 @@
-##  $Id: rmgroup.pl 7743 2008-04-06 10:04:43Z iulius $
+##  $Id: rmgroup.pl 9304 2011-08-04 22:10:47Z iulius $
 ##
 ##  rmgroup control message handler.
 ##
@@ -19,11 +19,15 @@ use strict;
 
 sub control_rmgroup {
     my ($par, $sender, $replyto, $site, $action, $log, $approved,
-        $headers, $body) = @_;
+        $article) = @_;
     my ($groupname) = @$par;
 
+    my $head = $article->head;
+    my @headers = split(/\r?\n/, $head->stringify);
+    my @body = split(/\r?\n/, $article->stringify_body);
+
     # Scan active to see what sort of change we are making.
-    open(ACTIVE, $inn::active) or logdie("Cannot open $inn::active: $!");
+    open(ACTIVE, $INN::Config::active) or logdie("Cannot open $INN::Config::active: $!");
     my @oldgroup;
     while (<ACTIVE>) {
         next unless /^(\Q$groupname\E)\s\d+\s\d+\s(\w)/;
@@ -33,58 +37,64 @@ sub control_rmgroup {
     close ACTIVE;
     my $status;
     if (not @oldgroup) {
-        $status = 'no change';
+        $status = 'not change';
     } elsif (not $approved) {
-        $status = 'unapproved';
+        $status = 'be unapproved';
     } else {
-        $status = 'removed';
+        $status = 'be removed';
     }
 
-    if ($action eq 'mail' and $status !~ /(no change|unapproved)/) {
+    if ($action eq 'mail' and $status !~ /(not change|be unapproved)/) {
         my $mail = sendmail("rmgroup $groupname $sender");
         print $mail <<END;
 $sender asks for $groupname
-to be $status.
+to $status.
 
 If this is acceptable, type:
-  $inn::newsbin/ctlinnd rmgroup $groupname
+  $INN::Config::newsbin/ctlinnd rmgroup $groupname
 
 And do not forget to remove the corresponding description, if any,
-from your newsgroups file.
+from your $INN::Config::pathdb/newsgroups file.
 
 The control message follows:
 
 END
-        print $mail map { s/^~/~~/; "$_\n" } @$headers;
+        print $mail map { s/^~/~~/; "$_\n" } @headers;
         print $mail "\n";
-        print $mail map { s/^~/~~/; "$_\n" } @$body;
+        print $mail map { s/^~/~~/; "$_\n" } @body;
         close $mail or logdie("Cannot send mail: $!");
     } elsif ($action eq 'log') {
         if ($log) {
             logger($log, "skipping rmgroup $groupname"
-                . " $sender (would be $status)", $headers, $body);
+                . " $sender (would $status)", $article);
         } else {
-            logmsg("skipping rmgroup $groupname $sender (would be $status)");
+            logmsg("skipping rmgroup $groupname $sender (would $status)");
         }
-    } elsif ($action eq 'doit' and $status !~ /(no change|unapproved)/) {
+    } elsif ($action eq 'doit' and $status !~ /(not change|be unapproved)/) {
         ctlinnd('rmgroup', $groupname);
         # Update newsgroups too.
-        shlock("$inn::locks/LOCK.newsgroups");
-        open(NEWSGROUPS, $inn::newsgroups)
-            or logdie("Cannot open $inn::newsgroups: $!");
-        my $tempfile = "$inn::newsgroups.$$";
+        my $lockfile = "$INN::Config::locks/LOCK.newsgroups";
+
+        # Acquire a lock.
+        INN::Utils::Shlock::lock($lockfile, 60) or logdie("Cannot create lockfile $lockfile");
+
+        open(NEWSGROUPS, $INN::Config::newsgroups)
+            or logdie("Cannot open $INN::Config::newsgroups: $!");
+        my $tempfile = "$INN::Config::newsgroups.$$";
         open(TEMPFILE, ">$tempfile") or logdie("Cannot open $tempfile: $!");
         while (<NEWSGROUPS>) {
             print TEMPFILE $_ if not /^\Q$groupname\E\s/;
         }
         close TEMPFILE;
         close NEWSGROUPS;
-        rename($tempfile, $inn::newsgroups)
+        rename($tempfile, $INN::Config::newsgroups)
             or logdie("Cannot rename $tempfile: $!");
-        unlink "$inn::locks/LOCK.newsgroups";
         unlink $tempfile;
 
-        logger($log, "rmgroup $groupname $status $sender", $headers, $body)
+        # Unlock.
+        INN::Utils::Shlock::unlock($lockfile);
+
+        logger($log, "rmgroup $groupname $status $sender", $article)
             if $log;
     }
 }
