@@ -1,4 +1,4 @@
-/*  $Id: makehistory.c 8894 2010-01-17 13:04:04Z iulius $
+/*  $Id: makehistory.c 9120 2010-09-26 11:37:05Z iulius $
 **
 **  Rebuild history/overview databases.
 */
@@ -95,6 +95,7 @@ bool WriteStdout = false;
 static char             BYTES[] = "Bytes";
 static char             DATE[] = "Date";
 static char             EXPIRES[] = "Expires";
+static char             INJECTIONDATE[] = "Injection-Date";
 static char             LINES[] = "Lines";
 static char		MESSAGEID[] = "Message-ID";
 static char		XREF[] = "Xref";
@@ -104,6 +105,7 @@ static size_t		ARTfieldsize;
 static ARTOVERFIELD     *Bytesp = (ARTOVERFIELD *)NULL;
 static ARTOVERFIELD     *Datep = (ARTOVERFIELD *)NULL;
 static ARTOVERFIELD     *Expp = (ARTOVERFIELD *)NULL;
+static ARTOVERFIELD     *InjectionDatep = (ARTOVERFIELD *)NULL;
 static ARTOVERFIELD     *Linesp = (ARTOVERFIELD *)NULL;
 static ARTOVERFIELD     *Msgidp = (ARTOVERFIELD *)NULL;
 static ARTOVERFIELD     *Xrefp = (ARTOVERFIELD *)NULL;
@@ -465,6 +467,9 @@ ARTreadschema(bool Overview)
             if (strncasecmp(extraoverview->strings[i],
                             EXPIRES, strlen(EXPIRES)) == 0)
                 Expp = fp;
+            if (strncasecmp(extraoverview->strings[i],
+                            INJECTIONDATE, strlen(INJECTIONDATE)) == 0)
+                InjectionDatep = fp;
             fp++;
         }
 
@@ -476,10 +481,14 @@ ARTreadschema(bool Overview)
         Missfieldsize++;
     if (Expp == (ARTOVERFIELD *)NULL)
         Missfieldsize++;
+    if (InjectionDatep == (ARTOVERFIELD *)NULL)
+        Missfieldsize++;
     if (Linesp == (ARTOVERFIELD *)NULL)
         Missfieldsize++;
     if (Msgidp == (ARTOVERFIELD *)NULL)
 	Missfieldsize++;
+    /* The fields in Missfields are not used in the overview (because they were not
+     * set before), so the values do not matter much (especially for NeedHeadername). */
     if (Missfieldsize > 0) {
 	Missfields = xmalloc(Missfieldsize * sizeof(ARTOVERFIELD));
         fp = Missfields;
@@ -502,7 +511,7 @@ ARTreadschema(bool Overview)
             Datep = fp++;
         }
         if (Expp == (ARTOVERFIELD *)NULL) {
-            fp->NeedHeadername = false;
+            fp->NeedHeadername = true;
             fp->Headername = xstrdup(EXPIRES);
             fp->HeadernameLength = strlen(EXPIRES);
             fp->Header = (char *)NULL;
@@ -510,6 +519,15 @@ ARTreadschema(bool Overview)
             fp->HeaderLength = 0;
             Expp = fp++;
         }
+        if (InjectionDatep == (ARTOVERFIELD *)NULL) {
+            fp->NeedHeadername = true;
+            fp->Headername = xstrdup(INJECTIONDATE);
+            fp->HeadernameLength = strlen(INJECTIONDATE);
+            fp->Header = (char *)NULL;
+            fp->HasHeader = false;
+            fp->HeaderLength = 0;
+            InjectionDatep = fp++;
+        }
         if (Linesp == (ARTOVERFIELD *)NULL) {
             fp->NeedHeadername = false;
             fp->Headername = xstrdup(LINES);
@@ -529,7 +547,7 @@ ARTreadschema(bool Overview)
             Msgidp = fp++;
         }
         if (Overview && Xrefp == (ARTOVERFIELD *)NULL) {
-	    fp->NeedHeadername = false;
+	    fp->NeedHeadername = true;
 	    fp->Headername = xstrdup(XREF);
 	    fp->HeadernameLength = strlen(XREF);
 	    fp->Header = (char *)NULL;
@@ -722,14 +740,24 @@ DoArt(ARTHANDLE *art)
      * newer than start time of makehistory. 
      */
 
-    if (!Datep->HasHeader) {
+    if (!InjectionDatep->HasHeader && !Datep->HasHeader) {
 	Posted = Arrived;
     } else {
-        buffer_set(&buffer, Datep->Header, Datep->HeaderLength);
-        buffer_append(&buffer, NUL, 1);
-        Posted = parsedate_rfc5322_lax(buffer.data);
-        if (Posted == (time_t) -1)
-	    Posted = Arrived;
+        if (InjectionDatep->HasHeader) {
+            buffer_set(&buffer, InjectionDatep->Header, InjectionDatep->HeaderLength);
+            buffer_append(&buffer, NUL, 1);
+            Posted = parsedate_rfc5322_lax(buffer.data);
+            /* If parsing failed for Injection-Date:, take the arrival time.
+             * Do not look at the Date: header (though we could). */
+            if (Posted == (time_t) -1)
+                Posted = Arrived;
+        } else {
+            buffer_set(&buffer, Datep->Header, Datep->HeaderLength);
+            buffer_append(&buffer, NUL, 1);
+            Posted = parsedate_rfc5322_lax(buffer.data);
+            if (Posted == (time_t) -1)
+                Posted = Arrived;
+        }
     }
 
     if (Expp->HasHeader) {
