@@ -1,4 +1,4 @@
-/*  $Id: feedone.c 6135 2003-01-19 01:15:40Z rra $
+/*  $Id: feedone.c 8921 2010-01-22 23:33:08Z iulius $
 **
 **  Connect to the NNTP server and feed one article.
 */
@@ -7,9 +7,9 @@
 #include "clibrary.h"
 #include <errno.h>
 
+#include "inn/libinn.h"
 #include "inn/messages.h"
-#include "libinn.h"
-#include "nntp.h"
+#include "inn/nntp.h"
 
 
 static FILE	*FromServer;
@@ -78,7 +78,6 @@ main(ac, av)
     FILE	*F;
     char	buff[BUFSIZ];
     char	*mesgid = NULL;
-    size_t      length;
     char	*p;
     char	*q;
     bool	PostMode;
@@ -104,10 +103,7 @@ main(ac, av)
 	    PostMode = true;
 	    break;
 	case 'r':			/* Random Message-ID	*/
-            length = snprintf(NULL, 0, "<%ld@%ld>", (long) getpid(),
-                              (long) time(NULL));
-            mesgid = xmalloc(length + 1);
-            snprintf(mesgid, length, "<%ld@%ld>", (long) getpid(),
+            xasprintf(&mesgid, "<%ld@%ld>", (long) getpid(),
                      (long) time(NULL));
 	    break;
 	case 't':
@@ -126,7 +122,7 @@ main(ac, av)
     /* Scan for the message-id. */
     if (mesgid == NULL) {
 	while (fgets(buff, sizeof buff, F) != NULL)
-	    if (strncasecmp(buff, MESGIDHDR, strlen(MESGIDHDR)) == 0) {
+	    if (strncmp(buff, MESGIDHDR, strlen(MESGIDHDR)) == 0) {
 		if ((p = strchr(buff, '<')) == NULL
                  || (q = strchr(p, '>')) == NULL)
                     die("bad message ID line");
@@ -139,7 +135,8 @@ main(ac, av)
     }
 
     /* Connect to the server. */
-    if (NNTPremoteopen(NNTP_PORT, &FromServer, &ToServer, buff) < 0
+    if (NNTPremoteopen(NNTP_PORT, &FromServer, &ToServer, buff,
+                       sizeof(buff)) < 0
      || FromServer == NULL
      || ToServer == NULL) {
 	if (buff[0])
@@ -150,11 +147,11 @@ main(ac, av)
     /* Does the server want this article? */
     if (PostMode) {
 	fprintf(ToServer, "post\r\n");
-	i = NNTP_START_POST_VAL;
+	i = NNTP_CONT_POST;
     }
     else {
 	fprintf(ToServer, "ihave %s\r\n", mesgid);
-	i = NNTP_SENDIT_VAL;
+	i = NNTP_CONT_IHAVE;
     }
     SafeFlush(ToServer);
     GetFromServer(buff, sizeof buff, "cannot offer article to server");
@@ -166,7 +163,7 @@ main(ac, av)
     /* Send the file over. */
     fseeko(F, 0, SEEK_SET);
     while (fgets(buff, sizeof buff, F) != NULL) {
-	if (strncasecmp(buff, MESGIDHDR, strlen(MESGIDHDR)) == 0) {
+	if (strncmp(buff, MESGIDHDR, strlen(MESGIDHDR)) == 0) {
 	    fprintf(ToServer, "%s %s\r\n", MESGIDHDR, mesgid);
 	    continue;
 	}
@@ -183,7 +180,7 @@ main(ac, av)
     /* How did the server respond? */
     GetFromServer(buff, sizeof buff,
 	"no reply from server after sending the article");
-    i = PostMode ? NNTP_POSTEDOK_VAL : NNTP_TOOKIT_VAL;
+    i = PostMode ? NNTP_OK_POST : NNTP_OK_IHAVE;
     if (atoi(buff) != i)
         sysdie("cannot send article to the server: %s", buff);
 
