@@ -1,4 +1,4 @@
-/*  $Id: inews.c 9024 2010-03-21 16:49:30Z iulius $
+/*  $Id: inews.c 9366 2011-08-22 08:51:14Z iulius $
 **
 **  Send an article (prepared by someone on the local site) to the
 **  master news server.
@@ -238,7 +238,7 @@ StripOffHeaders(char *article)
 	}
 
 	/* See if it's a known header. */
-	c = islower((unsigned char) *p) ? toupper(*p) : *p;
+	c = islower((unsigned char) *p) ? toupper((unsigned char) *p) : *p;
 	for (hp = Table; hp < ARRAY_END(Table); hp++)
 	    if (c == hp->Name[0]
 	     && p[hp->Size] == ':'
@@ -351,23 +351,25 @@ AnAdministrator(void)
     /* See if we are in the right group and examine process
      * supplementary groups, rather than the group(5) file entry.
      */
+#ifdef HAVE_GETGROUPS
     {
         int ngroups = getgroups(0, 0);
-        gid_t *groups, *gp;
+        GETGROUPS_T *groups, *gp;
         int rv;
         int rest;
 
-        groups = (gid_t *) xmalloc(ngroups * sizeof(*groups));
+        groups = (GETGROUPS_T *) xmalloc(ngroups * sizeof(GETGROUPS_T));
         if ((rv = getgroups(ngroups, groups)) < 0) {
             /* Silent failure; client doesn't have the group. */
             return false;
         }
         for (rest = ngroups, gp = groups; rest > 0; rest--, gp++) {
-            if (*gp == news_gid)
+            if (*gp == (GETGROUPS_T) news_gid)
                 return true;
         }
     }
-    
+#endif
+
     return false;
 }
 
