@@ -1,4 +1,4 @@
-/*  $Id: hisv6.c 7339 2005-06-20 03:16:41Z eagle $
+/*  $Id: hisv6.c 9094 2010-08-05 19:48:31Z iulius $
 **
 **  History v6 implementation against the history API.
 **
@@ -45,12 +45,12 @@
 #include "hisinterface.h"
 #include "hisv6.h"
 #include "hisv6-private.h"
-#include "dbz.h"
+#include "inn/dbz.h"
 #include "inn/innconf.h"
 #include "inn/timer.h"
 #include "inn/qio.h"
 #include "inn/sequence.h"
-#include "inndcomm.h"
+#include "inn/inndcomm.h"
 
 /*
 **  because we can only have one open dbz per process, we keep a
@@ -97,7 +97,8 @@ hisv6_splitline(const char *line, const char **error, HASH *hash,
 		 time_t *arrived, time_t *posted, time_t *expires,
 		 TOKEN *token)
 {
-    char *p = (char *)line;
+    const char *p = line;
+    char *end;
     unsigned long l;
     int r = 0;
 
@@ -122,7 +123,8 @@ hisv6_splitline(const char *line, const char **error, HASH *hash,
     }
 
     /* parse the arrived field */
-    l = strtoul(p + 1, &p, 10);
+    l = strtoul(p + 1, &end, 10);
+    p = end;
     if (l == ULONG_MAX) {
 	*error = "arrived timestamp out of range";
 	return -1;
@@ -144,7 +146,8 @@ hisv6_splitline(const char *line, const char **error, HASH *hash,
 	    if (expires)
 		*expires = 0;
 	} else {
-	    l = strtoul(p, &p, 10);
+	    l = strtoul(p, &end, 10);
+            p = end;
 	    if (l == ULONG_MAX) {
 		*error = "expires timestamp out of range";
 		return -1;
@@ -160,7 +163,8 @@ hisv6_splitline(const char *line, const char **error, HASH *hash,
 		*posted = 0;
 	} else {
 	    ++p;
-	    l = strtoul(p, &p, 10);
+	    l = strtoul(p, &end, 10);
+            p = end;
 	    if (l == ULONG_MAX) {
 		*error = "posted timestamp out of range";
 		return -1;
@@ -730,9 +734,20 @@ hisv6_formatline(char *s, const HASH *hash, time_t arrived,
     const char *hashtext = HashToText(*hash);
 
     if (token == NULL) {
-	i = snprintf(s, HISV6_MAXLINE, "[%s]%c%lu%c%c\n",
-		     hashtext, HISV6_FIELDSEP,
-		     (unsigned long)arrived, HISV6_SUBFIELDSEP, HISV6_NOEXP);
+        /* Only a line to remember an article.  We keep its arrival
+         * and posting time. */
+        if (posted <= 0) {
+            i = snprintf(s, HISV6_MAXLINE, "[%s]%c%lu%c%c\n",
+                         hashtext, HISV6_FIELDSEP,
+                         (unsigned long)arrived, HISV6_SUBFIELDSEP,
+                         HISV6_NOEXP);
+        } else {
+            i = snprintf(s, HISV6_MAXLINE, "[%s]%c%lu%c%c%c%lu\n",
+                         hashtext, HISV6_FIELDSEP,
+                         (unsigned long)arrived, HISV6_SUBFIELDSEP,
+                         HISV6_NOEXP, HISV6_SUBFIELDSEP,
+                         (unsigned long)posted);
+        }
     } else {
 	const char *texttok;
 
@@ -877,10 +892,11 @@ hisv6_write(void *history, const char *key, time_t arrived,
 
 
 /*
-**  remember a history entry, key, with arrival time arrived.
+**  Remember a history entry, key, with arrival time, and also
+**  posting time if known.
 */
 bool
-hisv6_remember(void *history, const char *key, time_t arrived)
+hisv6_remember(void *history, const char *key, time_t arrived, time_t posted)
 {
     struct hisv6 *h = history;
     HASH hash;
@@ -888,7 +904,7 @@ hisv6_remember(void *history, const char *key, time_t arrived)
 
     his_logger("HISwrite begin", S_HISwrite);
     hash = HashMessageID(key);
-    r = hisv6_writeline(h, &hash, arrived, 0, 0, NULL);
+    r = hisv6_writeline(h, &hash, arrived, posted, 0, NULL);
     his_logger("HISwrite end", S_HISwrite);
     return r;
 }
@@ -947,7 +963,7 @@ hisv6_replace(void *history, const char *key, time_t arrived,
 		do {
 		    n = pwrite(fileno(h->writefp), new, oldlen, offset);
 		} while (n == -1 && errno == EINTR);
-		if (n != oldlen) {
+		if ((size_t) n != oldlen) {
 		    char location[HISV6_MAX_LOCATION];
 
 		    hisv6_errloc(location, (size_t)-1, offset);
@@ -1117,7 +1133,7 @@ hisv6_expirecb(struct hisv6 *h, void *cookie, const HASH *hash,
 
     /* check if we've seen this message id already */
     if (hiscookie->new && dbzexists(*hash)) {
-	/* continue after duplicates, it serious, but not fatal */
+	/* continue after duplicates, it's serious, but not fatal */
 	hisv6_seterror(h, concat("duplicate message-id [",
 				 HashToText(*hash), "] in history ",
 				 hiscookie->new->histpath, NULL));
@@ -1136,18 +1152,24 @@ hisv6_expirecb(struct hisv6 *h, void *cookie, const HASH *hash,
 	    keep = (*hiscookie->cb.expire)(hiscookie->cookie,
 					   arrived, posted, expires,
 					   t);
-	    /* if the callback returns true, we should keep the
-	       token for the time being, else we just remember
-	       it */
+	    /* If the callback returns true, we should keep the
+	     * token for the time being, else we just remember
+	     * it. */
 	    if (keep == false) {
 		t = NULL;
-		posted = expires = 0;
+		expires = 0;
 	    }
 	} else {
 	    t = NULL;
 	}
+        /* When t is NULL (no token), the message-ID is removed from
+         * history when the posting time of the article is older than
+         * threshold, as set by the /remember/ line in expire.ctl.
+         * We keep the check for the arrival time because some entries
+         * might not have one. */
 	if (hiscookie->new &&
-	    (t != NULL || arrived >= hiscookie->threshold)) {
+	    (t != NULL || posted >= hiscookie->threshold
+             || (posted <= 0 && arrived >= hiscookie->threshold))) {
 	    r = hisv6_writeline(hiscookie->new, hash,
 				 arrived, posted, expires, t);
 	}
@@ -1237,7 +1259,7 @@ hisv6_expire(void *history, const char *path, const char *reason,
 	     bool (*exists)(void *, time_t, time_t, time_t, TOKEN *))
 {
     struct hisv6 *h = history, *hnew = NULL;
-    const char *nhistory = NULL;
+    char *nhistory = NULL;
     dbzoptions opt;
     bool r;
     struct hisv6_walkstate hiscookie;
@@ -1360,7 +1382,7 @@ hisv6_expire(void *history, const char *path, const char *reason,
     if (hnew && !hisv6_dispose(hnew))
 	r = false;
     if (nhistory && nhistory != path)
-	free((char *)nhistory);
+	free(nhistory);
     if (r == false && hiscookie.paused)
 	ICCgo(reason);
     return r;
