@@ -1,4 +1,4 @@
-/* $Id: hisv6.h 4959 2001-07-25 12:23:32Z alexk $
+/* $Id: hisv6.h 8575 2009-08-18 13:53:54Z iulius $
 **
 ** Internal history API interface exposed to HISxxx
 */
@@ -38,7 +38,7 @@ bool hisv6_walk(void *, const char *, void *,
 
 const char *hisv6_error(void *);
 
-bool hisv6_remember(void *, const char *key, time_t arrived);
+bool hisv6_remember(void *, const char *key, time_t arrived, time_t posted);
 
 bool hisv6_ctl(void *, int, void *);
 
