@@ -1,4 +1,4 @@
-/*  $Id: cc.c 9024 2010-03-21 16:49:30Z iulius $
+/*  $Id: cc.c 9417 2012-06-12 18:31:59Z iulius $
 **
 **  Routines for the control channel.
 **
@@ -259,6 +259,9 @@ CCaddhist(char *av[])
 	return "1 Bad posted date";
     Data.Posted = atol(av[3]);
 
+    /* Allow empty tokens, but not badly formatted tokens. */
+    if (*av[4] != '\0' && !IsToken(av[4]))
+        return "1 Bad token";
     token = TextToToken(av[4]);
     if (Mode == OMrunning)
 	ok = InndHisWrite(msgid, Data.Arrived, Data.Posted,
@@ -408,7 +411,7 @@ CCchgroup(char *av[])
     if (Rest[0] != NF_FLAG_ALIAS) {
 	Rest[1] = '\0';
 	if (isupper((unsigned char) Rest[0]))
-	    Rest[0] = tolower(Rest[0]);
+	    Rest[0] = tolower((unsigned char) Rest[0]);
     }
     return CCdochange(ngp, Rest);
 }
@@ -495,7 +498,7 @@ CCcheckfile(char *unused[])
   if (errors == 0)
     return NULL;
 
-  buffer_sprintf(&CCreply, false, "1 Found %d errors -- see syslog", errors);
+  buffer_sprintf(&CCreply, false, "1 Found %d error(s) -- see syslog", errors);
   return CCreply.data;
 }
 
@@ -680,7 +683,7 @@ CCgo(char *av[])
     Mode = OMrunning;
     ThrottledbyIOError = false;
 
-    if (NNRPReason && !innconf->readerswhenstopped) {
+    if (NNRPReason != NULL && !innconf->readerswhenstopped) {
 	av[0] = YES;
 	av[1] = p;
 	CCreaders(av);
@@ -999,7 +1002,7 @@ CCnewgroup(char *av[])
     if (Rest[0] != NF_FLAG_ALIAS) {
 	Rest[1] = '\0';
 	if (isupper((unsigned char) Rest[0]))
-	    Rest[0] = tolower(Rest[0]);
+	    Rest[0] = tolower((unsigned char) Rest[0]);
     }
 
     who = av[2];
@@ -1178,6 +1181,7 @@ CCblock(OPERATINGMODE NewMode, char *reason)
     if (ModeReason)
 	free(ModeReason);
     ModeReason = xstrdup(reason);
+    /* Disallow readers when the server is paused or throttled. */
     if (NNRPReason == NULL && !innconf->readerswhenstopped) {
 	av[0] = NO;
 	av[1] = ModeReason;
@@ -1311,6 +1315,7 @@ static const char *
 CCreload(char *av[])
 {
     const char *p;
+    const char *error;
 
 #ifdef DO_PERL
     static char BADPERLRELOAD[] = "1 Failed to define filter_art" ;
@@ -1323,6 +1328,9 @@ CCreload(char *av[])
 
     p = av[0];
     if (*p == '\0' || strcmp(p, "all") == 0) {
+        /* Check the syntax of the newsfeeds file before reloading. */
+        if ((error = CCcheckfile(NULL)) != NULL)
+            return error;
 	SITEflushall(false);
         if (Mode == OMrunning)
 	    InndHisClose();
@@ -1344,7 +1352,10 @@ CCreload(char *av[])
 	p = "all";
     }
     else if (strcmp(p, "active") == 0 || strcmp(p, "newsfeeds") == 0) {
-	SITEflushall(false);
+        /* Check the syntax of the newsfeeds file before reloading. */
+        if ((error = CCcheckfile(NULL)) != NULL)
+            return error;
+        SITEflushall(false);
 	ICDwrite();
 	ICDsetup(true);
     }
@@ -1403,8 +1414,11 @@ CCreload(char *av[])
 #ifdef DO_PERL
     else if (strcmp(p, "filter.perl") == 0) {
         path = concatpath(innconf->pathfilter, INN_PATH_PERL_FILTER_INND);
-        if (!PERLreadfilter(path, "filter_art"))
+        if (!PERLreadfilter(path, "filter_art")) {
+            free(path);
             return BADPERLRELOAD;
+        }
+        free(path);
     }
 #endif
 #ifdef DO_PYTHON
