@@ -1,61 +1,66 @@
-/*  $Id: icd.c,v 1.13 1998/10/21 07:28:06 brister Exp $
+/*  $Id: icd.c 8781 2009-11-15 09:24:59Z iulius $
 **
 **  Routines to read and write the active file.
 */
-#include <stdio.h>
-#include <sys/types.h>
-#include <netinet/in.h>
-#include "configdata.h"
+
+#include "config.h"
 #include "clibrary.h"
-#include "innd.h"
-#if	defined(ACT_MMAP)
-#include <sys/mman.h>
-#endif	/* defined(ACT_MMAP) */
+#include "portable/mmap.h"
 #include <sys/uio.h>
 
-typedef struct iovec	IOVEC;
+#include "inn/innconf.h"
+#include "inn/mmap.h"
+#include "innd.h"
+#include "inn/ov.h"
 
+/* If we fork and exec under Cygwin, children hold onto the mmap */
+/* of active, and Windows won't let us resize or replace it.     */
+#ifdef __CYGWIN__
+# undef HAVE_MMAP
+#endif
 
-STATIC char		*ICDactpath = NULL;
-STATIC char		*ICDactpointer;
-STATIC int		ICDactfd;
-STATIC int		ICDactsize;
+static char		*ICDactpath = NULL;
+static char		*ICDactpointer;
+static int		ICDactfd;
+static int		ICDactsize;
 
 
 /*
 **  Set and unset (or copy) IOVEC elements.  We make copies to
 **  avoid problems with mmap.
 */
-#if	defined(ACT_MMAP)
-void ICDiovset(IOVEC *iovp, char *base, int len) {
-    (iovp)->iov_len = len; 
-    (iovp)->iov_base = NEW(char, (iovp)->iov_len); 
-    (void)memcpy((POINTER)(iovp)->iov_base, (POINTER)base, 
-	    (SIZE_T)(iovp)->iov_len);
+#ifdef HAVE_MMAP
+static void
+ICDiovset(struct iovec *iovp, char *base, int len)
+{
+    iovp->iov_len = len; 
+    iovp->iov_base = xmalloc(iovp->iov_len); 
+    memcpy(iovp->iov_base, base, iovp->iov_len);
 }
-#define ICDiovrelease(iovp)		DISPOSE((iovp)->iov_base)
+#define ICDiovrelease(iovp)		free((iovp)->iov_base)
 
-#else
+#else /* !HAVE_MMAP */
 
 #define ICDiovset(iovp, base, len)	\
 	(iovp)->iov_base = base, (iovp)->iov_len = len
 #define ICDiovrelease(iovp)		/* NULL */
-#endif	/* defined(ACT_MMAP) */
+
+#endif /* HAVE_MMAP */
 
 
 /*
 **  Close the active file, releasing its resources.
 */
-STATIC void
-ICDcloseactive()
+static void
+ICDcloseactive(void)
 {
     if (ICDactpointer) {
-#if	defined(ACT_MMAP)
+#ifdef HAVE_MMAP
 	if (munmap(ICDactpointer, ICDactsize) < 0)
 	    syslog(L_ERROR, "%s cant munmap %s %m", LogName, ICDactpath);
 #else
-	DISPOSE(ICDactpointer);
-#endif	/* defined(ACT_MMAP) */
+	free(ICDactpointer);
+#endif
 	ICDactpointer = NULL;
 	if (close(ICDactfd) < 0) {
 	    syslog(L_FATAL, "%s cant close %s %m", LogName, ICDactpath);
@@ -69,11 +74,10 @@ ICDcloseactive()
 **  Set up the hash and in-core tables.
 */
 void
-ICDsetup(StartSites)
-    BOOL	StartSites;
+ICDsetup(bool StartSites)
 {
-    if (ICDneedsetup == TRUE) {
-	ICDneedsetup = FALSE;
+    if (ICDneedsetup == true) {
+	ICDneedsetup = false;
     }
     else {
 	ICDcloseactive();
@@ -83,7 +87,7 @@ ICDsetup(StartSites)
 	syslog(L_FATAL, "%s internal no control and/or junk group", LogName);
 	exit(1);
     }
-    if (innconf->usecontrolchan && NGfind("control.cancel") == NULL) {
+    if (NGfind("control.cancel") == NULL) {
 	syslog(L_FATAL, "%s internal no control.cancel group", LogName);
 	exit(1);
     }
@@ -99,11 +103,12 @@ ICDsetup(StartSites)
 **  Write out all in-core data.
 */
 void
-ICDwrite()
+ICDwrite(void)
 {
-    HISsync();
+    HISsync(History);
+    SMflushcacheddata(SM_ALL);
 
-    if (ICDactivedirty) {
+    if (ICDactivedirty != 0) {
 	ICDwriteactive();
 	ICDactivedirty = 0;
     }
@@ -120,7 +125,7 @@ ICDwrite()
 **  Close things down.
 */
 void
-ICDclose()
+ICDclose(void)
 {
     ICDwrite();
     ICDcloseactive();
@@ -130,36 +135,43 @@ ICDclose()
 /*
 **  Scan the active file, and renumber the min/max counts.
 */
-BOOL
-ICDrenumberactive()
+bool
+ICDrenumberactive(void)
 {
-    register int	i;
-    register NEWSGROUP	*ngp;
+    int	i;
+    NEWSGROUP	*ngp;
 
     for (i = nGroups, ngp = Groups; --i >= 0; ngp++)
 	if (!NGrenumber(ngp))
-	    return FALSE;
+	    return false;
     if (i < 0)
 	ICDwrite();
-    return TRUE;
+    return true;
 }
 
 
 /*
 **  Use writev() to replace the active file.
 */
-STATIC BOOL ICDwritevactive(IOVEC *vp, int vpcount)
+static bool
+ICDwritevactive(struct iovec *vp, int vpcount)
 {
     static char		*BACKUP = NULL;
     static char         *NEWACT = NULL;
     static char		WHEN[] = "backup active";
     int	                fd;
     int			oerrno;
+#ifdef __CYGWIN__
+    size_t		newactsize, padactsize, wrote;
+    struct iovec	*newvp;
+    char		*filler;
+    int			i;
+#endif
 
     if (BACKUP == NULL)
-	BACKUP = COPY(cpcatpath(innconf->pathdb, _PATH_OLDACTIVE));
+	BACKUP = concatpath(innconf->pathdb, INN_PATH_OLDACTIVE);
     if (NEWACT == NULL)
-	NEWACT = COPY(cpcatpath(innconf->pathdb, _PATH_NEWACTIVE));
+	NEWACT = concatpath(innconf->pathdb, INN_PATH_NEWACTIVE);
     /* Write the current file to a backup. */
     if (unlink(BACKUP) < 0 && errno != ENOENT) {
 	oerrno = errno;
@@ -175,7 +187,7 @@ STATIC BOOL ICDwritevactive(IOVEC *vp, int vpcount)
 	oerrno = errno;
 	syslog(L_ERROR, "%s cant write %s %m", LogName, BACKUP);
 	IOError(WHEN, oerrno);
-	(void)close(fd);
+	close(fd);
     }
     else if (close(fd) < 0) {
 	oerrno = errno;
@@ -183,13 +195,63 @@ STATIC BOOL ICDwritevactive(IOVEC *vp, int vpcount)
 	IOError(WHEN, oerrno);
     }
 
+#ifdef __CYGWIN__
+    /* If we are shrinking active, junk will be at the end between the */
+    /* writev and ftruncate.  Clobber it with values that overview and */
+    /* nnrpd can ignore. */
+    for (newactsize = 0, i = 0; i < vpcount; i++)
+	 newactsize += vp[i].iov_len;
+    if (newactsize < ICDactsize) {
+	 padactsize = ICDactsize - newactsize;
+	 newvp = xmalloc((vpcount + 1) * sizeof(struct iovec));
+	 for (i = 0; i < vpcount; i++)
+	      newvp[i] = vp[i];
+	 filler = xcalloc(padactsize, 1);
+	 *filler = '.';
+	 filler[padactsize - 1] = '\n';
+	 newvp[vpcount].iov_base = filler;
+	 newvp[vpcount].iov_len = padactsize;
+	 vpcount++;
+    }
+    else {
+	 padactsize = 0;
+	 newvp = vp;
+    }
+    oerrno = 0;
+    if (lseek(ICDactfd, 0, SEEK_SET) == -1) {
+        oerrno = errno;
+	syslog(L_ERROR, "%s cant rewind %s %m", LogName, ICDactpath);
+	IOError(WHEN, oerrno);
+	goto bailout;
+    }
+    if (xwritev(ICDactfd, newvp, vpcount) < 0) {
+	oerrno = errno;
+	syslog(L_ERROR, "%s cant write %s %m", LogName, ICDactpath);
+	IOError(WHEN, oerrno);
+	goto bailout;
+    }
+    if (newactsize < ICDactsize && ftruncate(ICDactfd, newactsize) != 0) {
+	oerrno = errno;
+	syslog(L_ERROR, "%s cant truncate %s", LogName, ICDactpath);
+    }
+
+bailout:
+    if (padactsize != 0) {
+	 free(filler);
+	 free(newvp);
+    }
+    if (oerrno != 0)
+	 return false;
+
+#else /* !__CYGWIN__, do it the Unix way. */
+
     /* Open the active file. */
     fd = open(NEWACT, O_WRONLY | O_TRUNC | O_CREAT, ARTFILE_MODE);
     if (fd < 0) {
 	oerrno = errno;
 	syslog(L_ERROR, "%s cant open %s %m", LogName, NEWACT);
 	IOError(WHEN, oerrno);
-	return FALSE;
+	return false;
     }
 
     /* Write it. */
@@ -197,14 +259,12 @@ STATIC BOOL ICDwritevactive(IOVEC *vp, int vpcount)
 	oerrno = errno;
 	syslog(L_ERROR, "%s cant write %s %m", LogName, NEWACT);
 	IOError(WHEN, oerrno);
-	(void)close(fd);
-	return FALSE;
+	close(fd);
+	return false;
     }
 
     /* Close it. */
-    (void)close(fd);
-    if (AmRoot)
-	xchown(NEWACT);
+    close(fd);
 
     /* Rename it to be the canonical active file */
     if (rename(NEWACT, ICDactpath) < 0) {
@@ -212,40 +272,44 @@ STATIC BOOL ICDwritevactive(IOVEC *vp, int vpcount)
 	syslog(L_ERROR, "%s cant rename %s to %s %m",
 	       LogName, NEWACT, ICDactpath);
 	IOError(WHEN, oerrno);
-	return FALSE;
+	return false;
     }
 
+#endif /* __CYGWIN__ */
+
     /* Invalidate in-core pointers. */
     ICDcloseactive();
 
     /* Restore in-core pointers. */
     if (Mode != OMrunning) {
-	ICDneedsetup = TRUE;
+	ICDneedsetup = true;
 	/* Force the active file into memory. */
 	NGparsefile();
     }
     else
-	ICDsetup(TRUE);
-    return TRUE;
+	ICDsetup(true);
+    return true;
 }
 
 
 /*
 **  Change the flag on a newsgroup.  Fairly easy.
 */
-BOOL
-ICDchangegroup(ngp, Rest)
-    NEWSGROUP		*ngp;
-    char		*Rest;
+bool
+ICDchangegroup(NEWSGROUP *ngp, char *Rest)
 {
     static char		NEWLINE[] = "\n";
-    register int	i;
-    IOVEC		iov[3];
-    BOOL		ret;
+    int                 i;
+    struct iovec	iov[3];
+    bool		ret;
+    char		*Name;
+    long		Last;
 
     /* Set up the scatter/gather vectors. */
     ICDiovset(&iov[0], ICDactpointer, ngp->Rest - ICDactpointer);
     ICDiovset(&iov[1], Rest, strlen(Rest));
+    Name = xstrdup(ngp->Name);
+    Last = ngp->Last;
     if (++ngp < &Groups[nGroups]) {
 	/* Not the last group, keep the \n from the next line. */
 	i = ngp->Start;
@@ -253,12 +317,20 @@ ICDchangegroup(ngp, Rest)
     }
     else {
 	/* Last group -- append a newline. */
-	ICDiovset(&iov[2], NEWLINE, STRLEN(NEWLINE));
+	ICDiovset(&iov[2], NEWLINE, strlen(NEWLINE));
     }
     ret = ICDwritevactive(iov, 3);
     ICDiovrelease(&iov[0]);
     ICDiovrelease(&iov[1]);
     ICDiovrelease(&iov[2]);
+
+    if (ret) {
+	if (innconf->enableoverview && !OVgroupadd(Name, 0, Last, Rest)) {
+	    free(Name);
+	    return false;
+	}
+    }
+    free(Name);
     return ret;
 }
 
@@ -266,27 +338,29 @@ ICDchangegroup(ngp, Rest)
 /*
 **  Add a newsgroup.  Append a line to the end of the active file and reload.
 */
-BOOL
-ICDnewgroup(Name, Rest)
-    char		*Name;
-    char		*Rest;
+bool
+ICDnewgroup(char *Name, char *Rest)
 {
     char		buff[SMBUF];
-    IOVEC		iov[2];
-    BOOL		ret;
+    struct iovec	iov[2];
+    bool		ret;
 
     /* Set up the scatter/gather vectors. */
-    if (strlen(Name) + strlen(Rest) > (SIZE_T)(SMBUF - 24)) {
+    if (strlen(Name) + strlen(Rest) > SMBUF - 24) {
 	syslog(L_ERROR, "%s too_long %s", LogName, MaxLength(Name, Name));
-	return FALSE;
+	return false;
     }
-    (void)sprintf(buff, "%s 0000000000 0000000001 %s\n", Name, Rest);
+    snprintf(buff, sizeof(buff), "%s 0000000000 0000000001 %s\n", Name, Rest);
     ICDiovset(&iov[0], ICDactpointer, ICDactsize);
     ICDiovset(&iov[1], buff, strlen(buff));
 
     ret = ICDwritevactive(iov, 2);
     ICDiovrelease(&iov[0]);
     ICDiovrelease(&iov[1]);
+    if (ret) {
+	if (innconf->enableoverview && !OVgroupadd(Name, 1, 0, Rest))
+	    return false;
+    }
     return ret;
 }
 
@@ -294,20 +368,35 @@ ICDnewgroup(Name, Rest)
 /*
 **  Remove a newsgroup.  Splice the line out of the active file and reload.
 */
-BOOL
-ICDrmgroup(ngp)
-    NEWSGROUP	*ngp;
+bool
+ICDrmgroup(NEWSGROUP *ngp)
 {
-    IOVEC	iov[2];
-    int		i;
-    BOOL	ret;
+    struct iovec iov[2];
+    int i;
+    bool ret;
+    char *Name;
 
+    /* Don't let anyone remove newsgroups that INN requires exist. */
+    if (strcmp(ngp->Name, "junk") == 0 || strcmp(ngp->Name, "control") == 0
+        || strcmp(ngp->Name, "control.cancel") == 0)
+        return false;
+    if (innconf->mergetogroups && strcmp(ngp->Name, "to") == 0)
+        return false;
+
+    Name = xstrdup(ngp->Name);
     /* If this is the first group in the file, write everything after. */
     if (ngp == &Groups[0]) {
 	i = ngp[1].Start;
 	ICDiovset(&iov[0], &ICDactpointer[i], ICDactsize - i);
 	ret = ICDwritevactive(iov, 1);
 	ICDiovrelease(&iov[0]);
+	if (ret) {
+	    if (innconf->enableoverview && !OVgroupdel(Name)) {
+		free(Name);
+		return false;
+	    }
+	}
+	free(Name);
 	return ret;
     }
 
@@ -318,6 +407,13 @@ ICDrmgroup(ngp)
     if (ngp == &Groups[nGroups - 1]) {
 	ret = ICDwritevactive(iov, 1);
 	ICDiovrelease(&iov[0]);
+	if (ret) {
+	    if (innconf->enableoverview && !OVgroupdel(Name)) {
+		free(Name);
+		return false;
+	    }
+	}
+	free(Name);
 	return ret;
     }
 
@@ -327,10 +423,17 @@ ICDrmgroup(ngp)
     ret = ICDwritevactive(iov, 2);
     ICDiovrelease(&iov[0]);
     ICDiovrelease(&iov[1]);
+    if (ret) {
+	if (innconf->enableoverview && !OVgroupdel(Name)) {
+	    free(Name);
+	    return false;
+	}
+    }
+    free(Name);
     return ret;
 }
 
-
+
 
 /*
 **  Open the active file and "map" it into memory.
@@ -346,36 +449,38 @@ ICDreadactive(endp)
 	return ICDactpointer;
     }
     if (ICDactpath == NULL) 
-	ICDactpath = COPY(cpcatpath(innconf->pathdb, _PATH_ACTIVE));
+	ICDactpath = concatpath(innconf->pathdb, INN_PATH_ACTIVE);
     if ((ICDactfd = open(ICDactpath, O_RDWR)) < 0) {
 	syslog(L_FATAL, "%s cant open %s %m", LogName, ICDactpath);
 	exit(1);
     }
-    CloseOnExec(ICDactfd, TRUE);
+    close_on_exec(ICDactfd, true);
+
+#ifdef HAVE_MMAP
 
-#if	defined(ACT_MMAP)
     if (fstat(ICDactfd, &Sb) < 0) {
 	syslog(L_FATAL, "%s cant fstat %d %s %m",
 	    LogName, ICDactfd, ICDactpath);
 	exit(1);
     }
     ICDactsize = Sb.st_size;
-    ICDactpointer = mmap((caddr_t)0, ICDactsize, PROT_READ|PROT_WRITE,
-			MAP__ARG, ICDactfd, (OFFSET_T)0);
+    ICDactpointer = mmap(NULL, ICDactsize, PROT_READ|PROT_WRITE,
+                         MAP_SHARED, ICDactfd, 0);
     if (ICDactpointer == (char *)-1) {
 	syslog(L_FATAL, "%s cant mmap %d %s %m",
 	    LogName, ICDactfd, ICDactpath);
 	exit(1);
     }
 
-#else
+#else /* !HAVE_MMAP */
 
     if ((ICDactpointer = ReadInDescriptor(ICDactfd, &Sb)) == NULL) {
 	syslog(L_FATAL, "%s cant read %s %m", LogName, ICDactpath);
 	exit(1);
     }
     ICDactsize = Sb.st_size;
-#endif	/* defined(ACT_MMAP) */
+
+#endif /* HAVE_MMAP */
 
     *endp = ICDactpointer + ICDactsize;
     return ICDactpointer;
@@ -386,25 +491,15 @@ ICDreadactive(endp)
 **  Write the active file out.
 */
 void
-ICDwriteactive()
+ICDwriteactive(void)
 {
-#if	defined(ACT_MMAP)
-#if defined (DO_MMAP_SYNC)
-#if defined (HAVE_MSYNC_3_ARG)
-    if (msync(ICDactpointer, ICDactsize, MS_ASYNC) < 0) {
-        syslog(L_FATAL, "%s msync failed %s %m", LogName, ICDactpath);
+#ifdef HAVE_MMAP
+    if (inn_msync_page(ICDactpointer, ICDactsize, MS_ASYNC) < 0) {
+        syslog(L_FATAL, "%s msync failed %s 0x%lx %d %m", LogName, ICDactpath, (unsigned long) ICDactpointer, ICDactsize);
         exit(1);
     }
-#else
-    if (msync(ICDactpointer, ICDactsize) < 0) {
-        syslog(L_FATAL, "%s msync failed %s %m", LogName, ICDactpath);
-        exit(1);
-    }
-#endif
-#endif
-#else
-
-    if (lseek(ICDactfd, 0L, SEEK_SET) == -1) {
+#else /* !HAVE_MMAP */
+    if (lseek(ICDactfd, 0, SEEK_SET) == -1) {
 	syslog(L_FATAL, "%s cant rewind %s %m", LogName, ICDactpath);
 	exit(1);
     }
@@ -412,5 +507,5 @@ ICDwriteactive()
 	syslog(L_FATAL, "%s cant write %s %m", LogName, ICDactpath);
 	exit(1);
     }
-#endif	/* defined(ACT_MMAP) */
+#endif /* HAVE_MMAP */
 }
