@@ -1,16 +1,16 @@
-/*  $Id: ng.c,v 1.15 1998/10/06 14:25:29 brister Exp $
+/*  $Id: ng.c 8426 2009-04-24 19:28:13Z iulius $
 **
 **  Routine for the in-core data structures for the active and newsfeeds
 **  files.
 */
-#include <stdio.h>
-#include <sys/types.h>
-#include <netinet/in.h>
-#include "configdata.h"
+
+#include "config.h"
 #include "clibrary.h"
+#include <dirent.h>
+
+#include "inn/innconf.h"
 #include "innd.h"
-#include <sys/mman.h>
-#include "mydir.h"
+#include "inn/ov.h"
 
 
 /*
@@ -39,43 +39,25 @@ typedef struct _NGHASH {
 } NGHASH;
 
 
-STATIC BUFFER	NGdirs;
-STATIC BUFFER	NGnames;
-STATIC NGHASH	NGHtable[NGH_SIZE];
-STATIC int	NGHbuckets;
-STATIC int	NGHcount;
+static struct buffer	NGnames;
+static NGHASH		NGHtable[NGH_SIZE];
+static int		NGHbuckets;
+static int		NGHcount;
+
 
-
 
 /*
 **  Sorting predicate for qsort call in NGparsefile.  Put newsgroups in
 **  rough order of their activity.  Will be better if we write a "counts"
 **  file sometime.
 */
-STATIC int
-NGcompare(p1, p2)
-    CPOINTER p1;
-    CPOINTER p2;
+static int
+NGcompare(const void *p1, const void *p2)
 {
-    NEWSGROUP	**ng1;
-    NEWSGROUP	**ng2;
+    const NEWSGROUP *const *n1 = p1;
+    const NEWSGROUP *const *n2 = p2;
 
-    ng1 = CAST(NEWSGROUP**, p1);
-    ng2 = CAST(NEWSGROUP**, p2);
-    return ng1[0]->Last - ng2[0]->Last;
-}
-
-
-/*
-**  Convert a newsgroup name into a directory name.
-*/
-STATIC void
-NGdirname(p)
-    register char	*p;
-{
-    for ( ; *p; p++)
-	if (*p == '.')
-	    *p = '/';
+    return n1[0]->Last - n2[0]->Last;
 }
 
 
@@ -84,67 +66,62 @@ NGdirname(p)
 **  not to write NUL's into the in-core copy, since we're either mmap(2)'d,
 **  or we want to just blat it out to disk later.
 */
-STATIC BOOL
-NGparseentry(ngp, p, end)
-    register NEWSGROUP		*ngp;
-    register char		*p;
-    register char		*end;
+static bool
+NGparseentry(NEWSGROUP *ngp, const char *p, char *end)
 {
-    register char		*q;
-    register unsigned int	j;
-    register NGHASH		*htp;
-    register NEWSGROUP		**ngpp;
-    register int		i;
+    char		*q;
+    unsigned int	j;
+    NGHASH		*htp;
+    NEWSGROUP		**ngpp;
+    int			i;
+    ARTNUM		lo;
 
     if ((q = strchr(p, ' ')) == NULL)
-	return FALSE;
+	return false;
     i = q - p;
 
     ngp->NameLength = i;
-    ngp->Name = &NGnames.Data[NGnames.Used];
-    (void)strncpy(ngp->Name, p, (SIZE_T)i);
+    ngp->Name = &NGnames.data[NGnames.used];
+    strncpy(ngp->Name, p, i);
     ngp->Name[i] = '\0';
-    NGnames.Used += i + 1;
-
-    ngp->Dir = &NGdirs.Data[NGdirs.Used];
-    (void)strncpy(ngp->Dir, p, (SIZE_T)i);
-    ngp->Dir[i] = '\0';
-    NGdirs.Used += i + 1;
-    NGdirname(ngp->Dir);
+    NGnames.used += i + 1;
 
     ngp->LastString = ++q;
     if ((q = strchr(q, ' ')) == NULL || q > end)
-	return FALSE;
+	return false;
     ngp->Lastwidth = q - ngp->LastString;
     if ((q = strchr(q, ' ')) == NULL || q > end)
-	return FALSE;
+	return false;
+    lo = (ARTNUM)atol(q + 1);
     if ((q = strchr(q + 1, ' ')) == NULL || q > end)
-	return FALSE;
+	return false;
     ngp->Rest = ++q;
     /* We count on atoi() to stop at the space after the digits! */
     ngp->Last = atol(ngp->LastString);
     ngp->nSites = 0;
-    ngp->Sites = NEW(int, NGHcount);
+    ngp->Sites = xmalloc(NGHcount * sizeof(int));
     ngp->nPoison = 0;
-    ngp->Poison = NEW(int, NGHcount);
+    ngp->Poison = xmalloc(NGHcount * sizeof(int));
     ngp->Alias = NULL;
 
     /* Find the right bucket for the group, make sure there is room. */
-    /* SUPPRESS 6 *//* Over/underflow from plus expression */
     NGH_HASH(ngp->Name, p, j);
     htp = NGH_BUCKET(j);
     for (p = ngp->Name, ngpp = htp->Groups, i = htp->Used; --i >= 0; ngpp++)
-	if (*p == ngpp[0]->Name[0] && EQ(p, ngpp[0]->Name)) {
+	if (*p == ngpp[0]->Name[0] && strcmp(p, ngpp[0]->Name) == 0) {
 	    syslog(L_ERROR, "%s duplicate_group %s", LogName, p);
-	    return FALSE;
+	    return false;
 	}
     if (htp->Used >= htp->Size) {
 	htp->Size += NGHbuckets;
-	RENEW(htp->Groups, NEWSGROUP*, htp->Size);
+        htp->Groups = xrealloc(htp->Groups, htp->Size * sizeof(NEWSGROUP *));
     }
     htp->Groups[htp->Used++] = ngp;
 
-    return TRUE;
+    if (innconf->enableoverview && !OVgroupadd(ngp->Name, lo, ngp->Last, ngp->Rest))
+	return false;
+
+    return true;
 }
 
 
@@ -152,30 +129,20 @@ NGparseentry(ngp, p, end)
 **  Parse the active file, building the initial Groups global.
 */
 void
-NGparsefile()
+NGparsefile(void)
 {
-    register char	*p;
-    register char	*q;
-    register int	i;
-    register BOOL	SawMe;
-    register NEWSGROUP	*ngp;
-    register NGHASH	*htp;
-    char		**strings;
-    char		*active;
-    char		*end;
+    char	*p;
+    char	*q;
+    int		i;
+    bool	SawMe;
+    NEWSGROUP	*ngp;
+    NGHASH	*htp;
+    char	**strings;
+    char	*active;
+    char	*end;
 
     /* If re-reading, remove anything we might have had. */
-    if (Groups) {
-	for (i = nGroups, ngp = Groups; --i >= 0; ngp++) {
-	    DISPOSE(ngp->Sites);
-	    DISPOSE(ngp->Poison);
-	}
-	DISPOSE(Groups);
-	DISPOSE(GroupPointers);
-	DISPOSE(NGdirs.Data);
-	DISPOSE(NGnames.Data);
-    }
-
+    NGclose();
 
     /* Get active file and space for group entries. */
     active = ICDreadactive(&end);
@@ -185,19 +152,16 @@ NGparsefile()
 	syslog(L_FATAL, "%s empty active file", LogName);
 	exit(1);
     }
-    Groups = NEW(NEWSGROUP, nGroups);
-    GroupPointers = NEW(NEWSGROUP*, nGroups);
+    Groups = xmalloc(nGroups * sizeof(NEWSGROUP));
+    GroupPointers = xmalloc(nGroups * sizeof(NEWSGROUP *));
 
-    /* Get space to hold copies of the names and the directory names.
-     * This might take more space than individually allocating each
-     * element, but it is definitely easier on the system. */
+    /* Get space to hold copies of the names.  This might take more space
+     * than individually allocating each element, but it is definitely easier
+     * on the system. */
     i = end - active;
-    NGdirs.Size = i;
-    NGdirs.Data = NEW(char, NGdirs.Size + 1);
-    NGdirs.Used = 0;
-    NGnames.Size = i;
-    NGnames.Data = NEW(char, NGnames.Size + 1);
-    NGnames.Used = 0;
+    NGnames.size = i;
+    NGnames.data = xmalloc(NGnames.size + 1);
+    NGnames.used = 0;
 
     /* Set up the default hash buckets. */
     NGHbuckets = nGroups / NGH_SIZE;
@@ -209,15 +173,15 @@ NGparsefile()
     else
 	for (i = NGH_SIZE, htp = NGHtable; --i >= 0; htp++) {
 	    htp->Size = NGHbuckets;
-	    htp->Groups = NEW(NEWSGROUP*, htp->Size);
+	    htp->Groups = xmalloc(htp->Size * sizeof(NEWSGROUP *));
 	    htp->Used = 0;
 	}
 
     /* Count the number of sites. */
-    SawMe = FALSE;
-    for (strings = SITEreadfile(TRUE), i = 0; (p = strings[i]) != NULL; i++)
+    SawMe = false;
+    for (strings = SITEreadfile(true), i = 0; (p = strings[i]) != NULL; i++)
 	if (*p == 'M' && *++p == 'E' && *++p == ':')
-	    SawMe = TRUE;
+	    SawMe = true;
     if (i == 0 || (i == 1 && SawMe)) {
 	syslog(L_ERROR, "%s bad_newsfeeds no feeding sites", LogName);
 	NGHcount = 1;
@@ -238,8 +202,7 @@ NGparsefile()
     /* Sort each bucket. */
     for (i = NGH_SIZE, htp = NGHtable; --i >= 0; htp++)
 	if (htp->Used > 1)
-	    qsort((POINTER)htp->Groups, (SIZE_T)htp->Used,
-		sizeof htp->Groups[0], NGcompare);
+	    qsort(htp->Groups, htp->Used, sizeof htp->Groups[0], NGcompare);
 
     /* Chase down any alias flags. */
     for (ngp = Groups, i = nGroups; --i >= 0; ngp++)
@@ -256,71 +219,104 @@ NGparsefile()
 	}
 }
 
+/*
+** Free allocated memory
+*/
+void
+NGclose(void)
+{
+    int		i;
+    NEWSGROUP	*ngp;
+    NGHASH	*htp;
+
+    if (Groups) {
+	for (i = nGroups, ngp = Groups; --i >= 0; ngp++) {
+	    free(ngp->Sites);
+	    free(ngp->Poison);
+	}
+	free(Groups);
+	Groups = NULL;
+	free(GroupPointers);
+	free(NGnames.data);
+    }
+
+    for (i = NGH_SIZE, htp = NGHtable; --i >= 0; htp++) {
+      htp->Size = NGHbuckets;
+      if (htp->Groups) {
+	free(htp->Groups);
+	htp->Used = 0;
+	htp->Groups = NULL;
+      }
+    }
+}
 
 /*
 **  Hash a newsgroup and see if we get it.
 */
 NEWSGROUP *
-NGfind(Name)
-    char			*Name;
+NGfind(const char *Name)
 {
-    register char		*p;
-    register int		i;
-    register unsigned int	j;
-    register NEWSGROUP		**ngp;
-    char			c;
-    NGHASH			*htp;
+    const char		*p;
+    int			i;
+    unsigned int	j;
+    NEWSGROUP		**ngp;
+    char		c;
+    NGHASH		*htp;
 
-    /* SUPPRESS 6 *//* Over/underflow from plus expression */
     NGH_HASH(Name, p, j);
     htp = NGH_BUCKET(j);
     for (c = *Name, ngp = htp->Groups, i = htp->Used; --i >= 0; ngp++)
-	if (c == ngp[0]->Name[0] && EQ(Name, ngp[0]->Name))
+	if (c == ngp[0]->Name[0] && strcmp(Name, ngp[0]->Name) == 0)
 	    return ngp[0];
     return NULL;
 }
 
 
 /*
-**  Split a newsgroups header line into the groups we get.  Return a
-**  point to static memory and clobber the argument along the way.
+**  Split a newsgroups header line into the groups we get.  Return the
+**  number of newsgroups.  ' ' and '\t' are dropped when copying.
 */
-char **
-NGsplit(p)
-    register char	*p;
+int
+NGsplit(char *p, int size, LISTBUFFER *list)
 {
-    static char		**groups;
-    static int		oldlength;
-    register char	**gp;
-    register int	i;
+  char		**gp, *q;
+  int		i;
 
-    /* Get an array of character pointers. */
-    i = strlen(p);
-    if (groups == NULL) {
-	groups = NEW(char*, i + 1);
-	oldlength = i;
-    }
-    else if (oldlength < i) {
-	RENEW(groups, char*, i + 1);
-	oldlength = i;
-    }
+  /* setup buffer */
+  SetupListBuffer(size, list);
 
-    /* Loop over text. */
-    for (gp = groups; *p; *p++ = '\0') {
-	/* Skip leading separators. */
-	for (; NG_ISSEP(*p); p++)
-	    continue;
-	if (*p == '\0')
-	    break;
+  /* loop over and copy */
+  for (i = 0, q = list->Data, gp = list->List ; *p ; p++, *q++ = '\0') {
+    /* skip leading separators and white spaces. */
+    for (; *p && (NG_ISSEP(*p) || ISWHITE(*p)) ; p++)
+      continue;
+    if (*p == '\0')
+      break;
 
-	/* Mark the start of the newsgroup, move to the end of it. */
-	for (*gp++ = p; *p && !NG_ISSEP(*p); p++)
-	    continue;
-	if (*p == '\0')
-	    break;
+    if (i == list->ListLength) {
+      list->ListLength += DEFAULTNGBOXSIZE;
+      list->List = xrealloc(list->List, list->ListLength * sizeof(char *));
+      gp = &list->List[i];
     }
-    *gp = NULL;
-    return groups;
+    /* mark the start of the newsgroup, move to the end of it while copying */
+    for (*gp++ = q, i++ ; *p && !NG_ISSEP(*p) && !ISWHITE(*p) ;) {
+      if (*p == ':')
+	/* reject if ':' is included */
+	return 0;
+      *q++ = *p++;
+      continue;
+    }
+    if (*p == '\0')
+      break;
+  }
+  *q = '\0';
+  if (i == list->ListLength) {
+    list->ListLength += DEFAULTNGBOXSIZE;
+    list->List = xrealloc(list->List, list->ListLength * sizeof(char *));
+    gp = &list->List[i];
+  }
+  *gp = NULL;
+  return i;
 }
 
 
@@ -330,30 +326,24 @@ NGsplit(p)
 static char		NORENUMBER[] = "%s cant renumber %s %s too wide";
 static char		RENUMBER[] = "%s renumber %s %s from %ld to %ld";
 
-BOOL NGrenumber(NEWSGROUP *ngp)
+bool
+NGrenumber(NEWSGROUP *ngp)
 {
-    DIR	                *dp;
-    DIRENTRY	        *ep;
+    int			low, high, count, flag;
     char	        *f2;
-    char	        *p;
     char		*f3;
     char		*f4;
     char		*start;
+    long		lomark, himark;
     long		l;
-    long		himark;
-    long		lomark;
     char		*dummy;
-    FILE                *fi;
-    OVERINDEX           index;
-    char                packed[OVERINDEXPACKSIZE];
-    struct stat		sb;
-    char		(*mapped)[][OVERINDEXPACKSIZE];
-    int			count;
+
+    if (!innconf->enableoverview) return true; /* can't do anything w/o overview */
 
     /* Get a valid offset into the active file. */
     if (ICDneedsetup) {
 	syslog(L_ERROR, "%s unsynched must reload before renumber", LogName);
-	return FALSE;
+	return false;
     }
     start = ICDreadactive(&dummy) + ngp->Start;
 
@@ -363,101 +353,57 @@ BOOL NGrenumber(NEWSGROUP *ngp)
      || (f4 = strchr(++f3, ' ')) == NULL) {
 	syslog(L_ERROR, "%s bad_format active %s",
 	    LogName, MaxLength(start, start));
-	return FALSE;
+	return false;
     }
-    himark = atol(f2);
-    lomark = himark + 1;
 
-    if (innconf->storageapi) {
-	p = NEW(char, strlen(innconf->pathoverview) + strlen(ngp->Dir) +
-					strlen(innconf->overviewname) + 32);
-	sprintf(p, "%s/%s/%s.index", innconf->pathoverview, ngp->Dir,
-					innconf->overviewname);
-	if ((fi = Fopen(p, "r", TEMPORARYOPEN)) == NULL) {
-	    DISPOSE(p);
-	} else {
-	    DISPOSE(p);
-	    if (innconf->overviewmmap) {
-		if (fstat(fileno(fi), &sb) < 0) {
-		    Fclose(fi);
-		    return TRUE;
-		}
-		count = sb.st_size / OVERINDEXPACKSIZE;
-		if (count == 0) {
-		    Fclose(fi);
-		    return TRUE;
-		}
-		if ((mapped = (char (*)[][OVERINDEXPACKSIZE])mmap((MMAP_PTR)0, count * OVERINDEXPACKSIZE,
-		    PROT_READ, MAP__ARG, fileno(fi), 0)) == (char (*)[][OVERINDEXPACKSIZE])-1) {
-		    Fclose(fi);
-		    return TRUE;
-		}
-		Fclose(fi);
-		/* assumes .overview.index is sorted */
-		UnpackOverIndex((*mapped)[0], &index);
-		if (index.artnum < lomark)
-		    lomark = index.artnum;
-		UnpackOverIndex((*mapped)[count-1], &index);
-		if (index.artnum > himark)
-		    himark = index.artnum;
-		if ((munmap((MMAP_PTR)mapped, count * OVERINDEXPACKSIZE)) < 0)
-		    return TRUE;
-	    } else {
-		while (fread(packed, OVERINDEXPACKSIZE, 1, fi) == 1) {
-		    UnpackOverIndex(packed, &index);
-		    if (index.artnum < lomark)
-			lomark = index.artnum;
-		    if (index.artnum > himark)
-			himark = index.artnum;
-		}
-		Fclose(fi);
-	    }
-	}
+    /* Check overview data for the group. */
+    if (!OVgroupstats(ngp->Name, &low, &high, &count, &flag)) return false;
+    if (count != 0) {
+	/* Non-empty group, so set low/high water marks from overview. */
+	lomark = low;
+	himark = high;
     } else {
-        /* Scan the directory. */
-	if ((dp = opendir(ngp->Dir)) != NULL) {
-	    while ((ep = readdir(dp)) != NULL) {
-		p = ep->d_name;
-		if (!CTYPE(isdigit, p[0]) || strspn(p, "0123456789") != strlen(p)
-		    || (l = atol(p)) == 0)
-		    continue;
-		if (l < lomark)
-		    lomark = l;
-		if (l > himark)
-		    himark = l;
-	    }
-	    (void)closedir(dp);
-	}
+        /* Empty group. */
+        himark = high;
+        lomark = himark + 1;
     }
     l = atol(f2);
-    if (himark != l) {
+    if (himark > l) {
 	syslog(L_NOTICE, RENUMBER, LogName, ngp->Name, "hi", l, himark);
+        /* f3 - f2 - 1 gives the length of the full high water mark
+         * in the active file. */
 	if (!FormatLong(f2, himark, f3 - f2 - 1)) {
 	    syslog(L_ERROR, NORENUMBER, LogName, ngp->Name, "hi");
-	    return FALSE;
+	    return false;
 	}
 	ngp->Last = himark;
 	ICDactivedirty++;
+    } else if (himark < l) {
+        syslog(L_NOTICE, "%s renumber %s hi not decreasing %ld to %ld",
+               LogName, ngp->Name, l, himark);
     }
     l = atol(f3);
     if (lomark != l) {
 	if (lomark < l)
 	    syslog(L_NOTICE, RENUMBER, LogName, ngp->Name, "lo", l, lomark);
+        /* f4 - f3 gives the length of the full low water mark
+         * in the active file.
+         * f4 points at the space ' ' before the newsgroup flag. */
 	if (!FormatLong(f3, lomark, f4 - f3)) {
 	    syslog(L_ERROR, NORENUMBER, LogName, ngp->Name, "lo");
-	    return FALSE;
+	    return false;
 	}
 	ICDactivedirty++;
     }
-    return TRUE;
+    return true;
 }
 
 /*
- * Set the low article count for the given group.
- * Like NGrenumber(), but we don't scan the spool,
- * and the himark is ignored.
- */
-BOOL NGlowmark(NEWSGROUP *ngp, long lomark)
+** Set the low article count for the given group.
+** Like NGrenumber(), but we don't query the overview database.
+*/
+bool
+NGlowmark(NEWSGROUP *ngp, long lomark)
 {
     long l;
     char *f2, *f3, *f4;
@@ -470,17 +416,31 @@ BOOL NGlowmark(NEWSGROUP *ngp, long lomark)
      || (f4 = strchr(++f3, ' ')) == NULL) {
         syslog(L_ERROR, "%s bad_format active %s",
             LogName, MaxLength(start, start));
-        return FALSE;
+        return false;
     }
+    /* Check whether the high water mark is more than the low water mark
+     * minus one (in the case of an empty newsgroup).  If not, we consider
+     * that the newsgroup is empty and we increase the high water mark. */
+    l = atol(f2);
+    if (l+1 < lomark) {
+        syslog(L_NOTICE, RENUMBER, LogName, ngp->Name, "hi", l, lomark-1);
+        if (!FormatLong(f2, lomark-1, f3 - f2 - 1)) {
+            syslog(L_ERROR, NORENUMBER, LogName, ngp->Name, "hi");
+            return false;
+        }
+        ngp->Last = lomark-1;
+        ICDactivedirty++;
+    }
+    /* Update the low water mark. */
     l = atol(f3);
     if (lomark != l) {
         if (lomark < l)
             syslog(L_NOTICE, RENUMBER, LogName, ngp->Name, "lo", l, lomark);
         if (!FormatLong(f3, lomark, f4 - f3)) {
             syslog(L_ERROR, NORENUMBER, LogName, ngp->Name, "lo");
-            return FALSE;
+            return false;
         }
         ICDactivedirty++;
     }
-    return TRUE;
+    return true;
 }
