@@ -1,40 +1,62 @@
-/*  $Id: python.c,v 1.23.2.1 2004/01/25 22:22:14 rra Exp $
+/*  $Id: python.c 9364 2011-08-21 23:23:26Z eagle $
 **
-**  python.c: Embed Python in the style of innd's TCL and Perl stuff.
-** 
-**  Written by G.J. Andruk <meowing@banet.net> patterned after
-**  TCL/Perl work by Bob Heiney and Christophe Wolfhugel and a whole
-**  bunch of other people mentioned in the docs and sources for the
-**  other filters.
+**  Embed Python in the style of innd's Perl stuff.
 **
-**  The astute reader may notice the commission of blatant atrocities
-**  against Python's OO model here.  Don't tell Guido.
+**  Written by G.J. Andruk <meowing@banet.net> patterned after Tcl/Perl work
+**  by Bob Heiney and Christophe Wolfhugel and a whole bunch of other people
+**  mentioned in the docs and sources for the other filters.
+**
+**  The astute reader may notice the commission of blatant atrocities against
+**  Python's OO model here.  Don't tell Guido.
+**
+**  A quick note regarding Python exceptions:  functions like
+**      PyObject_GetAttrString(PyObject *o, const char *attr_name)
+**  raise an exception when they fail, even though they return NULL.
+**  And as exceptions accumulate from caller to caller and so on,
+**  it generates weird issues with Python scripts afterwards.  So such
+**  uses should be checked before.  For instance with:
+**      PyObject_HasAttrString(PyObject *o, const char *attr_name).
 */
 
 #include "config.h"
+
+#ifdef DO_PYTHON
+
+/*  Python redefines _POSIX_C_SOURCE, so undef it to suppress warnings. */
+#undef _POSIX_C_SOURCE
+
+/*  Make "s#" use Py_ssize_t rather than int. */
+#define PY_SSIZE_T_CLEAN
+
+/*  Python.h must be included after having defined PY_SSIZE_T_CLEAN,
+ *  and before any standard headers are included (because Python may
+ *  define some pre-processor definitions which affect the standard
+ *  headers on some systems). */
+#include "Python.h"
+
+/*  Define Py_ssize_t when it does not exist (Python < 2.5.0). */
+#if PY_VERSION_HEX < 0x02050000
+  typedef int Py_ssize_t;
+#endif
+
 #include "clibrary.h"
 
 #include "inn/innconf.h"
+#include "inn/wire.h"
 #include "innd.h"
 
-
-#if defined(DO_PYTHON)
-
-#include "Python.h"
-
-
 bool		PythonFilterActive;
-char		*filterPath;	/* this gets set in art.c */
+char		*filterPath;	/* This gets set in art.c. */
 PyObject	*PYFilterObject = NULL;
 PyObject	*PYFilterModule = NULL;
 
-/* article filter bits and pieces */
+/*  Article filter bits and pieces. */
 PyObject	*PYheaders = NULL;
 PyObject	**PYheaditem;
 PyObject	**PYheadkey;
 PyObject	*PYpathkey, *PYlineskey, *PYbodykey;
 
-/* external functions */
+/*  External functions. */
 PyObject	*msgid_method = NULL;
 PyObject	*art_method = NULL;
 PyObject	*mode_method = NULL;
@@ -47,8 +69,7 @@ PyObject	*close_method = NULL;
 **  Turn filtering on or off.
 */
 void
-PYfilter(value)
-    bool value;
+PYfilter(bool value)
 {
     PythonFilterActive = value;
     syslog(L_NOTICE, "%s Python filtering %s", LogName,
@@ -58,14 +79,11 @@ PYfilter(value)
 
 
 /*
-**  Front end for PYfilter()
+**  Front end for PYfilter().
 */
 const char *
 PYcontrol(char **av)
 {
-    char        *p;
-    extern bool PythonFilterActive;
-
     switch (av[0][0]) {
     default:
         return "1 Bad flag";
@@ -87,18 +105,15 @@ PYcontrol(char **av)
 
 
 
-
 /*
 **  Reject articles we don't like.
 */
 char *
 PYartfilter(const ARTDATA *data, char *artBody, long artLen, int lines)
 {
-    const ARTHEADER *hp;
     const HDRCONTENT *hc = data->HdrContent;
     int		hdrnum;
     int		i;
-    char	*p, save;
     static char buf[256];
     PyObject	*result;
 
@@ -109,7 +124,6 @@ PYartfilter(const ARTDATA *data, char *artBody, long artLen, int lines)
     hdrnum = 0;
     for (i = 0 ; i < MAX_ARTHEADER ; i++) {
 	if (HDR_FOUND(i)) {
-	    hp = &ARTheaders[i];
 	    PYheaditem[hdrnum] = PyBuffer_FromMemory(HDR(i), HDR_LEN(i));
 	} else
 	    PYheaditem[hdrnum] = Py_None;
@@ -129,20 +143,21 @@ PYartfilter(const ARTDATA *data, char *artBody, long artLen, int lines)
     PyDict_SetItem(PYheaders, PYlineskey, PYheaditem[hdrnum++]);
 
     /* Now see if the filter likes it. */
-    result = PyObject_CallFunction(art_method, "O", PYheaders);
+    result = PyObject_CallFunction(art_method, (char *) "O", PYheaders);
     if ((result != NULL) && PyObject_IsTrue(result))
 	strlcpy(buf, PyString_AS_STRING(result), sizeof(buf));
     else
 	*buf = '\0';
     Py_XDECREF(result);
 
-    /* Clean up after ourselves */
+    /* Clean up after ourselves. */
     PyDict_Clear(PYheaders);
     for (i = 0; i < hdrnum; i++)
-	if (PYheaditem[i] != Py_None)
+	if (PYheaditem[i] != Py_None) {
 	    Py_DECREF(PYheaditem[i]);
+        }
 
-    if (*buf != '\0') 
+    if (*buf != '\0')
 	return buf;
     return NULL;
 }
@@ -150,12 +165,11 @@ PYartfilter(const ARTDATA *data, char *artBody, long artLen, int lines)
 
 
 /*
-**  Refuse message IDs offered thru CHECK or IHAVE that we don't like.
+**  Refuse message-IDs offered through CHECK, IHAVE or TAKETHIS that
+**  we don't like.
 */
 char *
-PYmidfilter(messageID, msglen)
-    char *messageID;
-    int msglen;
+PYmidfilter(char *messageID, int msglen)
 {
     static char		buf[256];
     PyObject		*result;
@@ -163,14 +177,15 @@ PYmidfilter(messageID, msglen)
     if (!PythonFilterActive || PYFilterObject == NULL || msgid_method == NULL)
 	return NULL;
 
-    result = PyObject_CallFunction(msgid_method, "s#", messageID, msglen);
+    result = PyObject_CallFunction(msgid_method, (char *) "s#", messageID,
+                                   msglen);
     if ((result != NULL) && PyObject_IsTrue(result))
 	strlcpy(buf, PyString_AS_STRING(result), sizeof(buf));
     else
 	*buf = '\0';
     Py_XDECREF(result);
 
-    if (*buf != '\0') 
+    if (*buf != '\0')
 	return buf;
     return NULL;
 }
@@ -196,6 +211,7 @@ PYmode(Mode, NewMode, reason)
     case OMrunning:	strlcpy(oldmode, "running", 10);	break;
     case OMpaused:	strlcpy(oldmode, "paused", 10);		break;
     case OMthrottled:	strlcpy(oldmode, "throttled", 10);	break;
+    case OMshutdown:    strlcpy(oldmode, "shutdown", 10);       break;
     }
 
     switch (NewMode) {
@@ -203,11 +219,12 @@ PYmode(Mode, NewMode, reason)
     case OMrunning:	strlcpy(newmode, "running", 10);	break;
     case OMpaused:	strlcpy(newmode, "paused", 10);		break;
     case OMthrottled:	strlcpy(newmode, "throttled", 10);	break;
+    case OMshutdown:    strlcpy(newmode, "shutdown", 10);       break;
     }
 
-    result = PyObject_CallFunction(mode_method, "sss",
+    result = PyObject_CallFunction(mode_method, (char *) "sss",
 				   oldmode, newmode, reason);
-    Py_DECREF(result);
+    Py_XDECREF(result);
 }
 
 
@@ -216,19 +233,21 @@ PYmode(Mode, NewMode, reason)
 **  Called by the external module so it can register itself with innd.
 */
 static PyObject *
-PY_set_filter_hook(dummy, args)
-    PyObject *dummy, *args;
+PY_set_filter_hook(PyObject *dummy UNUSED, PyObject *args)
 {
     PyObject	*result = NULL;
     PyObject	*temp;
 
-    if (PyArg_ParseTuple(args, "O:set_filter_hook", &temp)) {
+    /* set_filter_hook method should return a pointer to innd auth object. */
+    if (PyArg_ParseTuple(args, (char *) "O:set_filter_hook", &temp)) {
 	Py_XINCREF(temp);
 	Py_XDECREF(PYFilterObject);
 	PYFilterObject = temp;
 	Py_INCREF(Py_None);
 	result = Py_None;
     }
+
+    /* Return a pointer to innd auth method. */
     return result;
 }
 
@@ -238,13 +257,12 @@ PY_set_filter_hook(dummy, args)
 **  Allow external module to ask innd if an ID is in history.
 */
 static PyObject *
-PY_havehist(self, args)
-    PyObject *self, *args;
+PY_havehist(PyObject *self UNUSED, PyObject *args)
 {
     char	*msgid;
-    int		msgidlen;
+    Py_ssize_t  msgidlen;
 
-    if (!PyArg_ParseTuple(args, "s#", &msgid, &msgidlen))
+    if (!PyArg_ParseTuple(args, (char *) "s#", &msgid, &msgidlen))
 	return NULL;
 
     if (HIScheck(History, msgid))
@@ -258,14 +276,13 @@ PY_havehist(self, args)
 **  Allow external module to locally delete an article.
 */
 static PyObject *
-PY_cancel(self, args)
-    PyObject *self, *args;
+PY_cancel(PyObject *self UNUSED, PyObject *args)
 {
     char	*msgid;
-    int		msgidlen;
+    Py_ssize_t  msgidlen;
     char	*parambuf[2];
 
-    if (!PyArg_ParseTuple(args, "s#", &msgid, &msgidlen))
+    if (!PyArg_ParseTuple(args, (char *) "s#", &msgid, &msgidlen))
 	return NULL;
 
     parambuf[0]= msgid;
@@ -282,19 +299,18 @@ PY_cancel(self, args)
 **  Stuff an ID into history so that it will be refused later.
 */
 static PyObject *
-PY_addhist(self, args)
-    PyObject *self, *args;
+PY_addhist(PyObject *self UNUSED, PyObject *args)
 {
     char	*msgid;
-    int		msgidlen;
-    char	*articlepaths = "";
+    Py_ssize_t  msgidlen;
+    char	*articlepaths = (char *) "";
     char	tbuff[12];
     char	*parambuf[6];
 
-    if (!PyArg_ParseTuple(args, "s#", &msgid, &msgidlen))
+    if (!PyArg_ParseTuple(args, (char *) "s#", &msgid, &msgidlen))
 	return NULL;
 
-    snprintf(tbuff, sizeof(tbuff), "%d", time(NULL));
+    snprintf(tbuff, sizeof(tbuff), "%lu", (unsigned long) time(NULL));
 
     parambuf[0] = msgid;
     parambuf[1] = parambuf[2] = parambuf[3] = tbuff;
@@ -309,20 +325,18 @@ PY_addhist(self, args)
 
 
 /*
-**  Get a newsgroup's status flag (j, m, n, x, y, =other.group)
+**  Get a newsgroup's status flag (j, m, n, x, y, =other.group).
 */
 static PyObject *
-PY_newsgroup(self, args)
-    PyObject *self, *args;
+PY_newsgroup(PyObject *self UNUSED, PyObject *args)
 {
     char	*newsgroup;
-    int		nglen;
+    Py_ssize_t  nglen;
     NEWSGROUP	*ngp;
     char	*end;
-    char	*rest;
     int		size;
 
-    if (!PyArg_ParseTuple(args, "s#", &newsgroup, &nglen))
+    if (!PyArg_ParseTuple(args, (char *) "s#", &newsgroup, &nglen))
 	return NULL;
 
     ngp = NGfind(newsgroup);
@@ -337,9 +351,9 @@ PY_newsgroup(self, args)
 	size = end - ngp->Rest;
 
     /* If an alias is longer than this, active is probably broken. */
-    if (size > MAXHEADERSIZE) {
+    if (size > MED_BUFFER) {
 	syslog(L_ERROR, "too-long flag field in active for %s", newsgroup);
-	size = MAXHEADERSIZE;
+	size = MED_BUFFER;
     }
 
     return PyString_FromStringAndSize(ngp->Rest, size);
@@ -353,25 +367,24 @@ PY_newsgroup(self, args)
 **  for example, to compare two on-spool articles.
 */
 static PyObject *
-PY_head(self, args)
-    PyObject *self, *args;
+PY_head(PyObject *self UNUSED, PyObject *args)
 {
     char	*msgid;
-    int		msgidlen;
+    Py_ssize_t  msgidlen;
     char	*p;
     TOKEN	token;
     ARTHANDLE	*art;
     PyObject	*header;
-    int		headerlen;
+    size_t	headerlen;
 
-    if (!PyArg_ParseTuple(args, "s#", &msgid, &msgidlen))
+    if (!PyArg_ParseTuple(args, (char *) "s#", &msgid, &msgidlen))
 	return NULL;
 
     if (! HISlookup(History, msgid, NULL, NULL, NULL, &token))
-	return Py_BuildValue("s", "");	
+	return Py_BuildValue((char *) "s", "");	
     if ((art = SMretrieve(token, RETR_HEAD)) == NULL)
-	return Py_BuildValue("s", "");	
-    p = FromWireFmt(art->data, art->len, &headerlen);
+	return Py_BuildValue((char *) "s", "");	
+    p = wire_to_native(art->data, art->len, &headerlen);
     SMfreearticle(art);
     header = PyString_FromStringAndSize(p, headerlen);
     free(p);
@@ -385,25 +398,24 @@ PY_head(self, args)
 **  Return a whole article to the external module as a string.
 */
 static PyObject *
-PY_article(self, args)
-    PyObject *self, *args;
+PY_article(PyObject *self UNUSED, PyObject *args)
 {
     char	*msgid;
-    int		msgidlen;
+    Py_ssize_t  msgidlen;
     char	*p;
     TOKEN	token;
     ARTHANDLE	*arth;
     PyObject	*art;
-    int		artlen;
+    size_t	artlen;
 
-    if (!PyArg_ParseTuple(args, "s#", &msgid, &msgidlen))
+    if (!PyArg_ParseTuple(args, (char *) "s#", &msgid, &msgidlen))
 	return NULL;
 
     if (! HISlookup(History, msgid, NULL, NULL, NULL, &token))
-	return Py_BuildValue("s", "");
+	return Py_BuildValue((char *) "s", "");
     if ((arth = SMretrieve(token, RETR_ALL)) == NULL)
-	return Py_BuildValue("s", "");	
-    p = FromWireFmt(arth->data, arth->len, &artlen);
+	return Py_BuildValue((char *) "s", "");	
+    p = wire_to_native(arth->data, arth->len, &artlen);
     SMfreearticle(arth);
     art = PyString_FromStringAndSize(p, artlen);
     free(p);
@@ -419,19 +431,20 @@ PY_article(self, args)
 **  editor).
 */
 static PyObject *
-PY_syslog(self, args)
-    PyObject *self, *args;
+PY_syslog(PyObject *self UNUSED, PyObject *args)
 {
     char	*loglevel;
-    int		levellen;
+    Py_ssize_t  levellen;
     char	*logmsg;
-    int		msglen;
+    Py_ssize_t  msglen;
     int		priority;
 
-    if (!PyArg_ParseTuple(args, "s#s#",
+    /* Get loglevel and message. */
+    if (!PyArg_ParseTuple(args, (char *) "s#s#",
 			  &loglevel, &levellen, &logmsg, &msglen))
 	return NULL;
 
+    /* Assign syslog priority by abbreviated names. */
     switch (*loglevel) {
     default:		priority = LOG_NOTICE ;
     case 'd': case 'D': priority = LOG_DEBUG ;		break;
@@ -443,8 +456,10 @@ PY_syslog(self, args)
     case 'a': case 'A': priority = LOG_ALERT ;		break;
     }
 
+    /* Log the message. */
     syslog(priority, "python: %s", logmsg);
 
+    /* Return None. */
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -455,39 +470,40 @@ PY_syslog(self, args)
 **  Compute a hash digest for a string.
 */
 static PyObject *
-PY_hashstring(self, args)
-    PyObject *self, *args;
+PY_hashstring(PyObject *self UNUSED, PyObject *args)
 {
     char	*instring, *wpos, *p, *q;
     char	*workstring = NULL;
-    int		insize, worksize, newsize, i, wasspace;
+    Py_ssize_t  insize;
+    int         worksize, i;
+    bool        wasspace;
     int		lines = 0;
     HASH	myhash;
 
-    if (!PyArg_ParseTuple(args, "s#|i", &instring, &insize, &lines))
+    if (!PyArg_ParseTuple(args, (char *) "s#|i", &instring, &insize, &lines))
 	return NULL;
 
     /* If a linecount is provided, munge before hashing. */
     if (lines > 0) {
-	worksize = insize;
+	worksize = (int) insize;
 
-	/* chop leading whitespace */
-	for (p=instring ; worksize>0 && isspace(*p) ; p++) {
+	/* Chop leading whitespace. */
+	for (p=instring ; worksize>0 && isspace((unsigned char) *p) ; p++) {
 	    if (*p == '\n')
 		lines--;
 	    worksize--;
 	}
 	wpos = p;
 
-	/* and trailing */
-	for (p=&wpos[worksize] ; worksize>0 && isspace(*p) ; p--) {
+	/* And trailing. */
+	for (p=&wpos[worksize] ; worksize>0 && isspace((unsigned char) *p) ; p--) {
 	    if (*p == '\n')
 		lines--;
 	    worksize--;
 	}
 
-	/* chop last 3 lines if we have >= 5.  From above chop the
-	 * last line has no CR so we use 1 less here. */
+	/* Chop last 3 lines if we have >= 5.  From above chop the
+	 * last line which has no CR so we use 1 less here. */
 	if (lines >= 4) {
 	    for (i=0, p=wpos+worksize ; i<2 ; p--)
 		if (*p == '\n')
@@ -499,27 +515,27 @@ PY_hashstring(self, args)
 	 * do a copy because this is probably an original art
 	 * buffer. */
 	workstring =  memcpy(xmalloc(worksize), wpos, worksize);
-	newsize = wasspace = 0;
+        wasspace = false;
 	p = wpos;
 	q = workstring;
 	for (i=0 ; i<worksize ; i++) {
-	    if (isspace(*p)) {
-		if (!wasspace)
-		    *q++ = ' ';
-		wasspace = 1;
-	    }
-	    else {
-		*q++ = tolower(*p);
-		wasspace = 0;
-	    }
-	    p++;
-	}
+	    if (isspace((unsigned char) *p)) {
+                if (!wasspace) {
+                    *q++ = ' ';
+                }
+                wasspace = true;
+	    } else {
+                *q++ = tolower((unsigned char) *p);
+                wasspace = false;
+            }
+            p++;
+        }
 	worksize = q - workstring;
 	myhash = Hash(workstring, worksize);
 	free(workstring);
     }
     else
-	myhash = Hash(instring, insize);
+	myhash = Hash(instring, (int) insize);
 
     return PyString_FromStringAndSize((const char *)&myhash, sizeof(myhash));
 }
@@ -527,19 +543,25 @@ PY_hashstring(self, args)
 
 
 /*
-**  Make the internal INN module's functions visible to Python.
+**  Make the internal INN module's functions visible to Python.  Python
+**  annoyingly doesn't use const where appropriate in its structure
+**  definitions, so we have to add casts for all of the string parameters that
+**  we're initializing with constant strings.
 */
+#define METHOD(name, func, flags, help) \
+    { (char *)(name), (func), (flags), (char *)(help) }
+
 static PyMethodDef INNPyMethods[] = {
-    {"set_filter_hook", PY_set_filter_hook,	METH_VARARGS},
-    {"havehist",	PY_havehist,		METH_VARARGS},
-    {"addhist",		PY_addhist,		METH_VARARGS},
-    {"cancel",		PY_cancel,		METH_VARARGS},
-    {"newsgroup",	PY_newsgroup,		METH_VARARGS},
-    {"head",		PY_head,		METH_VARARGS},
-    {"article",		PY_article,		METH_VARARGS},
-    {"syslog",		PY_syslog,		METH_VARARGS},
-    {"hashstring",	PY_hashstring,		METH_VARARGS},
-    {NULL,		NULL}
+    METHOD("set_filter_hook", PY_set_filter_hook, METH_VARARGS, ""),
+    METHOD("havehist",        PY_havehist,        METH_VARARGS, ""),
+    METHOD("addhist",         PY_addhist,         METH_VARARGS, ""),
+    METHOD("cancel",          PY_cancel,          METH_VARARGS, ""),
+    METHOD("newsgroup",       PY_newsgroup,       METH_VARARGS, ""),
+    METHOD("head",            PY_head,            METH_VARARGS, ""),
+    METHOD("article",         PY_article,         METH_VARARGS, ""),
+    METHOD("syslog",          PY_syslog,          METH_VARARGS, ""),
+    METHOD("hashstring",      PY_hashstring,      METH_VARARGS, ""),
+    METHOD(NULL,              NULL,               0,            "")
 };
 
 
@@ -564,19 +586,31 @@ PYclose(void)
 **  Check that a method exists and is callable.	 Set a pointer to
 **  the corresponding PyObject, or NULL if not found.
 */
-void
-PYdefonemethod(methptr, methname)
-    PyObject	**methptr;
-    char	*methname;
+static void
+PYdefonemethod(PyObject **methptr, const char *methname)
 {
     Py_XDECREF(*methptr);
-    *methptr = PyObject_GetAttrString(PYFilterObject, methname);
+
+    /* We check with HasAttrString() the existence of the method because
+     * otherwise, in case it does not exist, an exception is raised by Python,
+     * although the result of the function is NULL. */
+    if (PyObject_HasAttrString(PYFilterObject, (char *) methname) == 1) {
+        /* Get a pointer to given method. */
+        *methptr = PyObject_GetAttrString(PYFilterObject, (char *) methname);
+    } else {
+        *methptr = NULL;
+    }
+
+    /* See if such method is defined. */
     if (*methptr == NULL)
 	syslog(L_NOTICE, "python method %s not found", methname);
-    else if (PyCallable_Check(*methptr) == 0) {
-	syslog(L_ERROR, "python object %s found but not a function", methname);
-	Py_DECREF(*methptr);
-	*methptr = NULL;
+    else {
+        /* See if it is callable. */
+        if (PyCallable_Check(*methptr) == 0) {
+            syslog(L_ERROR, "python object %s found but not a function", methname);
+            Py_DECREF(*methptr);
+            *methptr = NULL;
+        }
     }
 }
 
@@ -586,7 +620,7 @@ PYdefonemethod(methptr, methname)
 **  Look up the filter methods, so we will know what's available when
 **  innd wants to call them.
 */
-void
+static void
 PYdefmethods(void)
 {
     PYdefonemethod(&msgid_method, "filter_messageid");
@@ -599,7 +633,7 @@ PYdefmethods(void)
 
 
 /*
-**  Used by `ctlinnd reload filter.python'
+**  Used by "ctlinnd reload filter.python 'reason'".
 */
 int
 PYreadfilter(void)
@@ -618,20 +652,39 @@ PYreadfilter(void)
 	Py_XDECREF(result);
     }
 
+    /* We need to reimport the module before reloading it because otherwise,
+     * it might not be taken into account by Python.
+     * See Python API documentation:
+     *     If a module is syntactically correct but its initialization fails,
+     *     the first import statement for it does not bind its name locally,
+     *     but does store a (partially initialized) module object in
+     *     sys.modules.  To reload the module, you must first import it again
+     *     (this will bind the name to the partially initialized module object)
+     *     before you can reload() it.
+     */
+    PYFilterModule = PyImport_ImportModule((char *) INN_PATH_PYTHON_STARTUP_M);
+    if (PYFilterModule == NULL) {
+        syslog(L_ERROR, "failed to reimport external python module");
+    }
+
     if ((newmodule = PyImport_ReloadModule(PYFilterModule)) == NULL) {
 	syslog(L_ERROR, "cant reload python filter module");
 	PYfilter(false);
 	return 0;
     }
-    Py_DECREF(PYFilterModule);
+
+    Py_XDECREF(PYFilterModule);
     PYFilterModule = newmodule;
+
     if (PYFilterObject == NULL) {
 	syslog(L_ERROR, "python reload error, filter object not defined");
 	PYfilter(false);
 	return 0;
     }
+
     PYfilter(true);
     PYdefmethods();
+
     return 1;
 }
 
@@ -644,26 +697,31 @@ void
 PYsetup(void)
 {
     const ARTHEADER *hp;
-    int		hdrindex;
     size_t hdrcount;
 
+    /* Add path for nnrpd module.  The environment variable PYTHONPATH
+     * does it; one can also append innconf->pathfilter to sys.path once
+     * Python has been initialized. */
     setenv("PYTHONPATH", innconf->pathfilter, 1);
+
+    /* Load up the interpreter ;-O */
     Py_Initialize();
 
-    /* It makes Python sad when its stdout and stderr are closed. */
+    /* It makes Python sad when its stdout or stderr are closed. */
     if ((fileno(stdout) == -1) || (fileno(stderr) == -1))
 	PyRun_SimpleString
 	    ("import sys; sys.stdout=sys.stderr=open('/dev/null', 'a')");
 
+    /* See if Python initialized OK. */
     if (!Py_IsInitialized ()) {
 	syslog(L_ERROR, "python interpreter NOT initialized");
 	return;
     }
-    syslog(L_NOTICE, "python interpreter initialized OK");
 
-    Py_InitModule("INN", INNPyMethods);
+    /* Build a module interface to certain INN functions. */
+    Py_InitModule((char *) "INN", INNPyMethods);
 
-    PYFilterModule = PyImport_ImportModule(_PATH_PYTHON_STARTUP_M);
+    PYFilterModule = PyImport_ImportModule((char *) INN_PATH_PYTHON_STARTUP_M);
     if (PYFilterModule == NULL)
 	syslog(L_ERROR, "failed to import external python module");
 
@@ -677,8 +735,8 @@ PYsetup(void)
     }
 
     /* Grab space for these so we aren't forever recreating them.  We also
-       put the body and the line count into PYheaditem, so it needs to be
-       two elements longer than the total number of headers. */
+     * cut the body and the line count into PYheaditem, so it needs to be
+     * two elements longer than the total number of headers. */
     PYheaders = PyDict_New();
     hdrcount = ARRAY_END(ARTheaders) - ARTheaders;
     PYheaditem = xmalloc((hdrcount + 2) * sizeof(PyObject *));
@@ -690,6 +748,8 @@ PYsetup(void)
     PYpathkey = PyString_InternFromString("Path");
     PYlineskey = PyString_InternFromString("__LINES__");
     PYbodykey = PyString_InternFromString("__BODY__");
+
+    syslog(L_NOTICE, "python interpreter initialized OK");
 }
 
 #endif /* defined(DO_PYTHON) */
