@@ -1,4 +1,4 @@
-/*  $Id: endpoint.c 7738 2008-04-06 09:33:33Z iulius $
+/*  $Id: endpoint.c 9275 2011-07-17 19:09:26Z iulius $
 **
 **  The implementation of the innfeed EndPoint object class.
 **
@@ -36,7 +36,7 @@
 
 #include "inn/innconf.h"
 #include "inn/messages.h"
-#include "libinn.h"
+#include "inn/libinn.h"
 
 #include "buffer.h"
 #include "configfile.h"
@@ -106,7 +106,7 @@ extern const char *InputFile ;
 
 static EndPoint mainEndPoint ;
 static bool mainEpIsReg = false ;
-static unsigned int stdioFdMax = MAX_STDIO_FD ;
+unsigned int stdioFdMax = MAX_STDIO_FD ;
 
 time_t  PrivateTime;
 
@@ -228,12 +228,10 @@ EndPoint newEndPoint (int fd)
 
   if (fd > absHighestFd)
     {
-      static bool sizelogged = false ;
           
 #if defined (FD_SETSIZE)
-      if (fd >= FD_SETSIZE)
+      if ((unsigned int) fd >= FD_SETSIZE)
         {
-          sizelogged = true ;
           warn ("ME fd (%d) looks too big (%d -- FD_SETSIZE)", fd,
                 FD_SETSIZE) ;
           return NULL ;
@@ -241,7 +239,6 @@ EndPoint newEndPoint (int fd)
 #else
       if (fd > (sizeof (fd_set) * CHAR_BIT))
         {
-          sizelogged = true ;
           warn ("ME fd (%d) looks too big (%d -- sizeof (fd_set) * CHAR_BIT)",
                 fd, (sizeof (fd_set) * CHAR_BIT)) ;
           return NULL ;
@@ -348,19 +345,19 @@ int endPointFd (EndPoint endp)
 
 
 
-/* Request a read to be done next time there's data. The endpoint
- * ENDP is what will do the read. BUFF is the Buffer the data should
- * go into. FUNC is the callback function to call when the read is
- * complete. CLIENTDATA is the client data to pass back into the
+/* Request a read to be done next time there's data. The endpoint ENDP
+ * is what will do the read. BUFFERS is the array of Buffers the data
+ * should go into. FUNC is the callback function to call when the read
+ * is complete. CLIENTDATA is the client data to pass back into the
  * callback function. MINLEN is the minimum amount of data to
- * read. If MINLEN is 0 then then BUFF must be filled, otherwise at
+ * read. If MINLEN is 0 then then BUFFERS must be filled, otherwise at
  * least MINLEN bytes must be read.
  *
- * BUFF can be null, in which case no read is actually done, but the
+ * BUFFERS can be NULL, in which case no read is actually done, but the
  * callback function will be called still. This is useful for
  * listening sockets.
  *
- * Returns 0 if the read couln't be prepared (for example if a read
+ * Returns 0 if the read couldn't be prepared (for example if a read
  * is already outstanding).
  */
 
@@ -403,12 +400,12 @@ int prepareRead (EndPoint endp,
 
 
 
-/* Request a write to be done at a later point. ENDP is the EndPoint
- * to do the write. BUFF is the Buffer to write from. FUNC is the
- * function to call when the write is complete. CLIENTDATA is some
+/* Request a write to be done at a later point. ENDP is the EndPoint to
+ * do the write. BUFFERS is the array of Buffers to write from. FUNC is
+ * the function to call when the write is complete. CLIENTDATA is some
  * data to hand back to the callback function.
  *
- * If BUFF is null, then no write will actually by done, but the
+ * If BUFFERS is NULL, then no write will actually by done, but the
  * callback function will still be called. This is useful for
  * connecting sockets.
  *
@@ -530,7 +527,7 @@ TimeoutId prepareSleep (EndpTCB func, int timeToSleep, void *clientData)
 }
 
 
-/* Updates a an existing timeout request to go off TIMETOSLEEP seconds from
+/* Updates an existing timeout request to go off TIMETOSLEEP seconds from
    now, or queues a new request.  Always returns a new ID. */
 TimeoutId updateSleep (TimeoutId tid, EndpTCB func, int timeToSleep,
                        void *clientData) 
@@ -586,7 +583,6 @@ void Run (void)
   fd_set rSet ;
   fd_set wSet ;
   fd_set eSet ;
-  unsigned long last_summary = 0 ;
 
   keepSelecting = 1 ;
   xsignal (SIGPIPE, pipeHandler) ;
@@ -633,17 +629,10 @@ void Run (void)
       TMRstop(TMR_IDLE);
 
       timePasses () ;
-      if (innconf->timer)
-        {
-	  unsigned long now = TMRnow () ;
-	  if (last_summary == 0 
-	      || (long) (now - last_summary) > (innconf->timer * 1000))
-	    {
-	      TMRsummary ("ME", timer_name) ;
-	      last_summary = now;
-	    }
-	}
-      
+      if (innconf->timer != 0 && TMRnow() > innconf->timer * 1000) {
+          TMRsummary ("ME", timer_name);
+      }
+
       if (sval == 0 && twait == NULL)
         die ("No fd's ready and no timeouts") ;
       else if (sval < 0 && errno == EINTR)
@@ -1394,22 +1383,11 @@ static void doTimeout (void)
 
 
 
-
 #if defined (WANT_MAIN)
 
-
 #define BUFF_SIZE 100
 
-
-void timerCallback (void *cd) ;
-void timerCallback (void *cd)
-{
-  d_printf (1,"Callback \n") ;
-}
-
-  
-void lineIsWritten (EndPoint ep, IoStatus status, Buffer *buffer, void *data);
-void lineIsWritten (EndPoint ep, IoStatus status, Buffer *buffer, void *data)
+static void lineIsWritten (EndPoint ep, IoStatus status, Buffer *buffers, void *d UNUSED)
 {
   int i ;
   
@@ -1424,12 +1402,12 @@ void lineIsWritten (EndPoint ep, IoStatus status, Buffer *buffer, void *data)
       errno = oldErrno ;
     }
 
-  for (i = 0 ; buffer && buffer [i] ; i++)
-    delBuffer (buffer [i]) ;
+  for (i = 0 ; buffers && buffers[i] ; i++) {
+    delBuffer (buffers[i]);
+  }
 }
 
-void lineIsRead (EndPoint myEp, IoStatus status, Buffer *buffer, void *data);
-void lineIsRead (EndPoint myEp, IoStatus status, Buffer *buffer, void *d)
+static void lineIsRead (EndPoint myEp, IoStatus status, Buffer *buffers, void *d UNUSED)
 {
   Buffer *writeBuffers, *readBuffers ;
   Buffer newBuff1, newBuff2 ;
@@ -1456,13 +1434,13 @@ void lineIsRead (EndPoint myEp, IoStatus status, Buffer *buffer, void *d)
     }
   
   
-  data = bufferBase (buffer[0]) ;
-  len = bufferDataSize (buffer[0]) ;
+  data = bufferBase (buffers[0]);
+  len = bufferDataSize (buffers[0]);
   
   if (data [len - 1] == '\r' || data [len - 1] == '\n')
-    bufferDecrDataSize (buffer [0],1) ;
+    bufferDecrDataSize (buffers[0], 1);
   if (data [len - 1] == '\r' || data [len - 1] == '\n')
-    bufferDecrDataSize (buffer [0],1) ;
+    bufferDecrDataSize (buffers[0], 1);
 
   data [len] = '\0' ;
   
@@ -1473,34 +1451,29 @@ void lineIsRead (EndPoint myEp, IoStatus status, Buffer *buffer, void *d)
   newInputBuffer = newBuffer (BUFF_SIZE) ;
   
   p = bufferBase (newBuff1) ; 
-  strcpy (p, "Thanks for that \"") ;
+  strlcpy (p, "Thanks for that \"", bufferSize (newBuff1)) ;
   bufferSetDataSize (newBuff1,strlen (p)) ;
   
   p = bufferBase (newBuff2) ;
-  strcpy (p,"\" very tasty\n") ;
+  strlcpy (p,"\" very tasty\n", bufferSize (newBuff2)) ;
   bufferSetDataSize (newBuff2,strlen (p)) ;
 
-  writeBuffers = makeBufferArray (newBuff1,buffer[0],newBuff2,NULL) ;
+  writeBuffers = makeBufferArray (newBuff1, buffers[0], newBuff2, NULL);
   readBuffers = makeBufferArray (newInputBuffer,NULL) ;
   
-  prepareWrite (myEp,writeBuffers,lineIsWritten,NULL) ;
-  prepareRead (myEp,readBuffers,lineIsRead,NULL,1) ;
-
-#if 0
-  myEp->registerWake (&timerCallback,theTime() + 7,0) ;
-#endif
+  prepareWrite(myEp, writeBuffers, NULL, lineIsWritten, NULL);
+  prepareRead(myEp, readBuffers, lineIsRead, NULL, 1);
 }
 
 
-static void printDate (TimeoutId tid, void *data) ;
 static void printDate (TimeoutId tid, void *data)
 {
-  time_t t ;
+  char dateString[30];
+  const time_t t = theTime() ;
 
-  t = theTime() ;
-  
-  d_printf (1,"Timeout (%d) time now is %ld %s",
-           (int) tid,(long) t,ctime(&t)) ;
+  timeToString (t, dateString, sizeof (dateString)) ;
+  d_printf (1,"Timeout (%d) time now is %ld %s\n",
+            (int) tid, (long) t, dateString) ;
 
   if (timeoutQueue == NULL) 
     {
@@ -1512,13 +1485,13 @@ static void printDate (TimeoutId tid, void *data)
 
 TimeoutId rm ;
 
-static void Timeout (TimeoutId tid, void *data) ;
 static void Timeout (TimeoutId tid, void *data)
 {
   static int seeded ;
   static int howMany ;
   static int i ;
-  time_t t = theTime() ;
+  char dateString[30];
+  const time_t t = theTime() ;
 
   if ( !seeded )
     {
@@ -1526,9 +1499,10 @@ static void Timeout (TimeoutId tid, void *data)
       seeded = 1 ;
     }
 
-  d_printf (1,"Timeout (%d) time now is %ld %s",
-           (int) tid, (long) t,ctime(&t)) ;
-  
+  timeToString (t, dateString, sizeof (dateString)) ;
+  d_printf (1,"Timeout (%d) time now is %ld %s\n",
+            (int) tid, (long) t, dateString) ;
+
   if (timeoutQueue != NULL && timeoutQueue->next != NULL)
     d_printf (1,"%s timeout id %d\n",
              (removeTimeout (rm) ? "REMOVED" : "FAILED TO REMOVE"), rm) ;
@@ -1550,14 +1524,13 @@ static void Timeout (TimeoutId tid, void *data)
 }
 
 
-void newConn (EndPoint ep, IoStatus status, Buffer *buffer, void *d) ;
-void newConn (EndPoint ep, IoStatus status, Buffer *buffer, void *d)
+static void newConn (EndPoint ep, IoStatus status UNUSED, Buffer *buffers UNUSED, void *d UNUSED)
 {
   EndPoint newEp ;
   struct sockaddr_in in ;
   Buffer *readBuffers ;
   Buffer newBuff = newBuffer (BUFF_SIZE) ;
-  int len = sizeof (in) ;
+  socklen_t len = sizeof (in);
   int fd ;
 
   memset (&in, 0, sizeof (in)) ;
@@ -1572,11 +1545,11 @@ void newConn (EndPoint ep, IoStatus status, Buffer *buffer, void *d)
   
   newEp = newEndPoint (fd) ;
 
-  prepareRead (ep, NULL, newConn,NULL,0) ;
+  prepareRead(ep, NULL, newConn, NULL, 0);
 
   readBuffers = makeBufferArray (newBuff,NULL) ;
 
-  prepareRead (newEp, readBuffers, lineIsRead, NULL, 1) ;
+  prepareRead(newEp, readBuffers, lineIsRead, NULL, 1);
 
   d_printf (1,"Set up a new connection\n");
 }
@@ -1591,7 +1564,7 @@ int main (int argc, char **argv)
   time_t t = theTime() ;
 
 
-  program = strrchr (argv[0],'/') ;
+  char * program = strrchr (argv[0],'/') ;
 
   if (!program)
     program = argv [0] ;
@@ -1621,7 +1594,7 @@ int main (int argc, char **argv)
   
   accConn = newEndPoint (accFd) ;
 
-  prepareRead (accConn,NULL,newConn,NULL,0) ;
+  prepareRead(accConn, NULL, newConn, NULL, 0);
 
   prepareSleep (Timeout,5,(void *) 0x10) ;
 
@@ -1668,31 +1641,18 @@ void setSigHandler (int signum, void (*ptr)(int))
 static void handleSignals (void)
 {
   int i ;
-#if defined(USE_SIGVEC)
-  int mask ;
-#endif
 
   for (i = 1; i < NSIG; i++)
     {
       if (sigFlags[i])
         {
-#if defined(USE_SIGACTION)
+#if defined(HAVE_SIGACTION)
           sigset_t set, oset ;
       
           if (sigemptyset (&set) != 0 || sigaddset (&set, i) != 0)
             die ("sigemptyset or sigaddset failed") ;
           if (sigprocmask (SIG_BLOCK, &set, &oset) != 0)
             die ("sigprocmask failed: %s", strerror(errno)) ;
-#elif defined(USE_SIGVEC)
-# ifndef sigmask
-#  define sigmask(s)    (1 << ((s) - 1))
-# endif
-          int mask ;
-          
-          mask = sigblock (sigmask(i)) ;
-#elif defined(USE_SIGSET)
-          if (sighold (i) != 0)
-            die ("sighold failed: %s", strerror(errno)) ;
 #else
       /* hope for the best */
 #endif
@@ -1704,14 +1664,9 @@ static void handleSignals (void)
               sigHandlers[i] != SIG_DFL)
             (sigHandlers[i])(i) ;
             
-#if defined(USE_SIGACTION)
+#if defined(HAVE_SIGACTION)
           if (sigprocmask (SIG_SETMASK, &oset, (sigset_t *)NULL) != 0)
             die ("sigprocmask failed: %s", strerror(errno)) ;
-#elif defined(USE_SIGVEC)
-          sigsetmask (mask) ;
-#elif defined(USE_SIGSET)
-          if (sigrelse (i) != 0)
-            die ("sigrelse failed: %s", strerror(errno)) ;
 #else
           /* hope for the best */
 #endif
