@@ -1,4 +1,4 @@
-/*  $Id: innlistener.c 9024 2010-03-21 16:49:30Z iulius $
+/*  $Id: innlistener.c 9275 2011-07-17 19:09:26Z iulius $
 **
 **  The implementation of the innfeed InnListener class.
 **
@@ -54,7 +54,7 @@ static FILE *droppedFp = NULL ;
 static long droppedCount = 0 ;
 static int droppedFileCount = 0 ;
 static char *dropArtFile = NULL ;
-static bool fastExit = false ;
+bool fastExit = false ;
 
 extern const char *pidFile ;
 extern const char *InputFile ;
