@@ -1,4 +1,4 @@
-/*  $Id: conffile.c 6733 2004-05-16 23:01:23Z rra $
+/*  $Id: conffile.c 7585 2006-11-21 09:37:51Z eagle $
 **
 **  Routines for reading in incoming.conf-style config files.
 */
@@ -6,7 +6,7 @@
 #include "config.h"
 #include "clibrary.h"
 #include "conffile.h"
-#include "libinn.h"
+#include "inn/libinn.h"
 
 static int getconfline(CONFFILE *F, char *buffer, int length) {
   if (F->f) {
@@ -111,7 +111,7 @@ static char *CONFgetword(CONFFILE *F)
   return (word);
 }
 
-CONFFILE *CONFfopen(char *filename)
+CONFFILE *CONFfopen(const char *filename)
 {
   FILE *f;
   CONFFILE *ret;
