@@ -1,87 +1,87 @@
-/*
-  dbz.c  V6.1.1
-
-Copyright 1988 Jon Zeeff (zeeff@b-tech.ann-arbor.mi.us)
-You can use this code in any manner, as long as you leave my name on it
-and don't hold me responsible for any problems with it.
-
-Hacked on by gdb@ninja.UUCP (David Butler); Sun Jun  5 00:27:08 CDT 1988
-
-Various improvments + INCORE by moraes@ai.toronto.edu (Mark Moraes)
-
-Major reworking by Henry Spencer as part of the C News project.
-
-Minor lint and CodeCenter (Saber) fluff removal by Rich $alz (March, 1991).
-Non-portable CloseOnExec() calls added by Rich $alz (September, 1991).
-Added "writethrough" and tagmask calculation code from
-<rob@violet.berkeley.edu> and <leres@ee.lbl.gov> by Rich $alz (December, 1992).
-Merged in MMAP code by David Robinson, formerly <david@elroy.jpl.nasa.gov>
-now <david.robinson@sun.com> (January, 1993).
-
-Major reworking by Clayton O'Neill (coneill@oneill.net).  Removed all the
-C News and backwards compatible cruft.  Ripped out all the tagmask stuff
-and replaced it with hashed .pag entries.  This removes the need for
-base file access.  Primary bottleneck now appears to be the hash
-algorithm and search().  You can change DBZ_INTERNAL_HASH_SIZE in
-dbz.h to increase the size of the stored hash.
-
-These routines replace dbm as used by the usenet news software
-(it's not a full dbm replacement by any means).  It's fast and
-simple.  It contains no AT&T code.
-
-The dbz database exploits the fact that when news stores a <key,value>
-tuple, the `value' part is a seek offset into a text file, pointing to
-a copy of the `key' part.  This avoids the need to store a copy of
-the key in the dbz files. 
-
-The basic format of the database is two hash tables, each in it's own
-file. One contains the offsets into the history text file , and the
-other contains a hash of the message id.  A value is stored by
-indexing into the tables using a hash value computed from the key;
-collisions are resolved by linear probing (just search forward for an
-empty slot, wrapping around to the beginning of the table if
-necessary).  Linear probing is a performance disaster when the table
-starts to get full, so a complication is introduced.  Each file actually
-contains one *or more* tables, stored sequentially in the files, and
-the length of the linear-probe sequences is limited.  The search (for
-an existing item or an empy slot always starts in the first table of
-the hash file, and whenever MAXRUN probes have been done in table N,
-probing continues in table N+1.  It is best not to overflow into more
-than 1-2 tables or else massive performance degradation may occur.
-Choosing the size of the database is extremely important because of this.
-
-The table size is fixed for any particular database, but is determined
-dynamically when a database is rebuilt.  The strategy is to try to pick
-the size so the first table will be no more than 2/3 full, that being
-slightly before the point where performance starts to degrade.  (It is
-desirable to be a bit conservative because the overflow strategy tends
-to produce files with holes in them, which is a nuisance.)
-
-Tagged hash + offset fuzzy technique merged by Sang-yong Suh (Nov, 1997)
-
-Fixed a bug handling larger than 1Gb history offset by Sang-yong Suh(1998)
-Similar fix was suggested by Mike Hucka <hucka@umich.edu> (Jan, 1998) for
-dbz-3.3.2.
-
-Limited can't tag warnings once per dbzinit() by Sang-yong Suh (May, 1998)
-
+/*  $Id: dbz.c 9325 2011-08-06 20:55:43Z iulius $
+**
+**  dbz database implementation V6.1.1
+**
+**  Copyright 1988 Jon Zeeff (zeeff@b-tech.ann-arbor.mi.us)
+**  You can use this code in any manner, as long as you leave my name on it
+**  and don't hold me responsible for any problems with it.
+**  
+**  Hacked on by gdb@ninja.UUCP (David Butler); Sun Jun  5 00:27:08 CDT 1988
+**  Various improvments + INCORE by moraes@ai.toronto.edu (Mark Moraes)
+**  Major reworking by Henry Spencer as part of the C News project.
+**  
+**  Minor lint and CodeCenter (Saber) fluff removal by Rich $alz
+**    (March, 1991).
+**  Non-portable CloseOnExec() calls added by Rich $alz (September, 1991).
+**  Added "writethrough" and tagmask calculation code from
+**    <rob@violet.berkeley.edu> and <leres@ee.lbl.gov> by Rich $alz
+**    (December, 1992).
+**  Merged in MMAP code by David Robinson, formerly <david@elroy.jpl.nasa.gov>
+**    now <david.robinson@sun.com> (January, 1993).
+**  
+**  Major reworking by Clayton O'Neill (coneill@oneill.net).  Removed all the
+**  C News and backwards compatible cruft.  Ripped out all the tagmask stuff
+**  and replaced it with hashed .pag entries.  This removes the need for base
+**  file access.  Primary bottleneck now appears to be the hash algorithm and
+**  search().  You can change DBZ_INTERNAL_HASH_SIZE in dbz.h to increase the
+**  size of the stored hash.
+**  
+**  These routines replace dbm as used by the usenet news software (it's not a
+**  full dbm replacement by any means).  It's fast and simple.  It contains no
+**  AT&T code.
+**  
+**  The dbz database exploits the fact that when news stores a <key,value>
+**  tuple, the "value" part is a seek offset into a text file, pointing to a
+**  copy of the "key" part.  This avoids the need to store a copy of the key
+**  in the dbz files.
+**  
+**  The basic format of the database is two hash tables, each in it's own
+**  file.  One contains the offsets into the history text file, and the other
+**  contains a hash of the message id.  A value is stored by indexing into the
+**  tables using a hash value computed from the key; collisions are resolved
+**  by linear probing (just search forward for an empty slot, wrapping around
+**  to the beginning of the table if necessary).  Linear probing is a
+**  performance disaster when the table starts to get full, so a complication
+**  is introduced.  Each file actually contains one *or more* tables, stored
+**  sequentially in the files, and the length of the linear-probe sequences is
+**  limited.  The search (for an existing item or an empy slot always starts
+**  in the first table of the hash file, and whenever MAXRUN probes have been
+**  done in table N, probing continues in table N+1.  It is best not to
+**  overflow into more than 1-2 tables or else massive performance degradation
+**  may occur.  Choosing the size of the database is extremely important
+**  because of this.
+**  
+**  The table size is fixed for any particular database, but is determined
+**  dynamically when a database is rebuilt.  The strategy is to try to pick
+**  the size so the first table will be no more than 2/3 full, that being
+**  slightly before the point where performance starts to degrade.  (It is
+**  desirable to be a bit conservative because the overflow strategy tends to
+**  produce files with holes in them, which is a nuisance.)
+**  
+**  Tagged hash + offset fuzzy technique merged by Sang-yong Suh (Nov, 1997)
+**  
+**  Fixed a bug handling larger than 1Gb history offset by Sang-yong Suh
+**  (1998) Similar fix was suggested by Mike Hucka <hucka@umich.edu> (Jan,
+**  1998) for dbz-3.3.2.
+**  
+**  Limited can't tag warnings once per dbzinit() by Sang-yong Suh (May, 1998)
 */
 
 #include "config.h"
 #include "clibrary.h"
+#include "portable/mmap.h"
 #include <ctype.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <netinet/in.h>
-#include <sys/mman.h>
 #include <sys/stat.h>
 #include <sys/time.h>
 
-#include "dbz.h"
+#include "inn/dbz.h"
 #include "inn/messages.h"
 #include "inn/innconf.h"
 #include "inn/mmap.h"
-#include "libinn.h"
+#include "inn/libinn.h"
 
 /* Needed on AIX 4.1 to get fd_set and friends. */
 #ifdef HAVE_SYS_SELECT_H
@@ -865,7 +865,7 @@ dbzsync(void)
     if (!putcore(&idxtab) || !putcore(&etab)) {
 #endif
 	warn("dbzsync: putcore failed");
-	ret = false;
+	ret = false;;
     }
 
     if (putconf(dirf, &conf) < 0)
@@ -948,7 +948,7 @@ dbzfetch(const HASH key, off_t *value)
     nb2r = sizeof(buffer) - 1;
 
     while ((offset = search(&srch)) != NOTFOUND) {
-	debug("got 0x%lx", key_ptr);
+	debug("got 0x%lx", offset);
 
 	/* fetch the key */
 	offset <<= conf.dropbits;
@@ -1037,7 +1037,10 @@ dbzfetch(const HASH key, off_t *value)
 
 /*
  * dbzstore - add an entry to the database
- * returns true for success and false for failure
+ *
+ * returns DBZSTORE_OK     for success
+ *         DBZSTORE_EXISTS for existing entries (duplicates)
+ *         DBZSTORE_ERROR  for other failure
  */
 DBZSTORE_RESULT
 dbzstore(const HASH key, off_t data)
@@ -1136,9 +1139,8 @@ getconf(FILE *df, dbzconfig *cp)
 	cp->tagshift = TAGSHIFT;
 	cp->dropbits = 0;
 	cp->lenfuzzy = 0;
-	debug("getconf: defaults (%ld, %c, (0x%lx/0x%lx<<%d %d))",
-              cp->tsize, cp->casemap, cp->tagenb, 
-              cp->tagmask, cp->tagshift, cp->dropbits);
+	debug("getconf: defaults (%ld, (0x%lx/0x%lx<<%d %d))",
+              cp->tsize, cp->tagenb, cp->tagmask, cp->tagshift, cp->dropbits);
 	return true;
     }
 
@@ -1248,8 +1250,10 @@ getcore(hash_table *tab)
     char *it;
     ssize_t nread;
     size_t i;
-    struct stat st;
     size_t length = conf.tsize * tab->reclen;
+#ifdef HAVE_MMAP
+    struct stat st;
+#endif
 
     if (tab->incore == INCORE_MMAP) {
 #if defined(HAVE_MMAP)
@@ -1257,7 +1261,7 @@ getcore(hash_table *tab)
 	    syswarn("dbz: getcore: fstat failed");
 	    return false;
 	}
-	if (length > st.st_size) {
+	if ((off_t) length > st.st_size) {
 	    /* file too small; extend it */
 	    if (ftruncate(tab->fd, length) == -1) {
 		syswarn("dbz: getcore: ftruncate failed");
@@ -1270,7 +1274,7 @@ getcore(hash_table *tab)
 	    syswarn("dbz: getcore: mmap failed");
 	    return false;
 	}
-#if defined (MADV_RANDOM) && defined(HAVE_MADVISE)
+#ifdef MADV_RANDOM
 	/* not present in all versions of mmap() */
 	madvise(it, length, MADV_RANDOM);
 #endif
@@ -1311,7 +1315,7 @@ putcore(hash_table *tab)
 	    return true;
 	nonblocking(tab->fd, false);
 	size = tab->reclen * conf.tsize;
-	result = xpwrite(tab->fd, tab->core, size, 0);
+        result = xpwrite(tab->fd, tab->core, size, 0);
 	if (result < 0 || (size_t) result != size) {
 	    nonblocking(tab->fd, options.nonblock);
 	    return false;
@@ -1408,7 +1412,7 @@ search(searcher *sp)
 	    taboffset = sp->tabno * conf.tsize;
 	}
 	sp->place = ((sp->shorthash + sp->run) % conf.tsize) + taboffset;
-	debug("search @ %ld", place);
+	debug("search @ %ld", sp->place);
 
 	/* get the tagged value */
 	if ((options.pag_incore != INCORE_NO) && (sp->place < conf.tsize)) {
@@ -1542,7 +1546,7 @@ set(searcher *sp, hash_table *tab, void *value)
 	debug("set: incore");
 	if (tab->incore == INCORE_MMAP) {
 	    if (innconf->nfswriter) {
-		inn_mapcntl(where, tab->reclen, MS_ASYNC);
+		inn_msync_page(where, tab->reclen, MS_ASYNC);
 	    }
 	    return true;
 	}
@@ -1667,7 +1671,7 @@ usage(void)
 #ifdef DO_TAGGED_HASH
     fprintf(stderr, "  -i       initialize history. deletes .pag files\n");
 #else
-    fprintf(stderr, "  -i       initialize history. deletes .exists and .index files\n");
+    fprintf(stderr, "  -i       initialize history. deletes .hash and .index files\n");
 #endif
     fprintf(stderr, "  -n or m  use INCORE_NO, INCORE_MMAP. default = INCORE_MEM\n");
     fprintf(stderr, "  -N       using nfswriter mode\n");
