@@ -1,4 +1,4 @@
-/*  $Id: localopen.c 6155 2003-01-19 19:58:25Z rra $
+/*  $Id: localopen.c 8921 2010-01-22 23:33:08Z iulius $
 **
 */
 
@@ -8,11 +8,11 @@
 #include <sys/socket.h>
 
 #include "inn/innconf.h"
-#include "libinn.h"
-#include "nntp.h"
-#include "paths.h"
+#include "inn/libinn.h"
+#include "inn/nntp.h"
+#include "inn/paths.h"
 
-#if HAVE_UNIX_DOMAIN_SOCKETS
+#ifdef HAVE_UNIX_DOMAIN_SOCKETS
 # include <sys/un.h>
 #endif
 
@@ -22,7 +22,7 @@
 **  create stdio FILE's for talking to it.  Return -1 on error.
 */
 int
-NNTPlocalopen(FILE **FromServerp, FILE **ToServerp, char *errbuff)
+NNTPlocalopen(FILE **FromServerp, FILE **ToServerp, char *errbuff, size_t len)
 {
 #if	defined(HAVE_UNIX_DOMAIN_SOCKETS)
     int			i;
@@ -30,10 +30,15 @@ NNTPlocalopen(FILE **FromServerp, FILE **ToServerp, char *errbuff)
     int			oerrno;
     struct sockaddr_un	server;
     FILE		*F;
-    char		mybuff[NNTP_STRLEN + 2];
+    char		mybuff[NNTP_MAXLEN_COMMAND + 2];
     char		*buff;
 
-    buff = errbuff ? errbuff : mybuff;
+    if (errbuff)
+        buff = errbuff;
+    else {
+        buff = mybuff;
+        len = sizeof(mybuff);
+    }
     *buff = '\0';
 
     /* Create a socket. */
@@ -45,7 +50,7 @@ NNTPlocalopen(FILE **FromServerp, FILE **ToServerp, char *errbuff)
     server.sun_family = AF_UNIX;
     strlcpy(server.sun_path, innconf->pathrun, sizeof(server.sun_path));
     strlcat(server.sun_path, "/", sizeof(server.sun_path));
-    strlcat(server.sun_path, _PATH_NNTPCONNECT, sizeof(server.sun_path));
+    strlcat(server.sun_path, INN_PATH_NNTPCONNECT, sizeof(server.sun_path));
     if (connect(i, (struct sockaddr *)&server, SUN_LEN(&server)) < 0) {
 	oerrno = errno;
 	close(i);
@@ -60,14 +65,14 @@ NNTPlocalopen(FILE **FromServerp, FILE **ToServerp, char *errbuff)
 	errno = oerrno;
 	return -1;
     }
-    if (fgets(buff, sizeof mybuff, F) == NULL) {
+    if (fgets(buff, len, F) == NULL) {
 	oerrno = errno;
 	fclose(F);
 	errno = oerrno;
 	return -1;
     }
     j = atoi(buff);
-    if (j != NNTP_POSTOK_VAL && j != NNTP_NOPOSTOK_VAL) {
+    if (j != NNTP_OK_BANNER_POST && j != NNTP_OK_BANNER_NOPOST) {
 	fclose(F);
 	/* This seems like a reasonable error code to use... */
 	errno = EPERM;
@@ -84,6 +89,6 @@ NNTPlocalopen(FILE **FromServerp, FILE **ToServerp, char *errbuff)
     return 0;
 #else
     return NNTPconnect("127.0.0.1", innconf->port, FromServerp, ToServerp,
-                       errbuff);
+                       errbuff, len);
 #endif	/* defined(HAVE_UNIX_DOMAIN_SOCKETS) */
 }
