@@ -1,46 +1,45 @@
-#include <stdio.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <errno.h>
-#include "configdata.h"
+#include "config.h"
 #include "clibrary.h"
-#include "libinn.h"
+#include <errno.h>
+#include <sys/stat.h>
+
+#include "inn/libinn.h"
 
 /*
-**  Try to make one directory.  Return FALSE on error.
+**  Try to make one directory.  Return false on error.
 */
-STATIC BOOL MakeDir(char *Name)
+static bool MakeDir(char *Name)
 {
     struct stat		Sb;
 
     if (mkdir(Name, GROUPDIR_MODE) >= 0) {
-	return TRUE;
+	return true;
     }
 
     /* See if it failed because it already exists. */
     if (stat(Name, &Sb) >= 0 && S_ISDIR(Sb.st_mode)) {
 	errno = 0;
-	return TRUE;
+	return true;
     }
-    return FALSE;
+    return false;
 }
 
 
 /*
 **  Given a directory, comp/foo/bar, create that directory and all
-**  intermediate directories needed.  Return TRUE if ok, else FALSE.
+**  intermediate directories needed.  Return true if ok, else false.
 */
-BOOL MakeDirectory(char *Name, BOOL Recurse)
+bool MakeDirectory(char *Name, bool Recurse)
 {
     char	        *p;
-    BOOL		made;
+    bool		made;
 
     /* Optimize common case -- parent almost always exists. */
     if (MakeDir(Name))
-	return TRUE;
+	return true;
 
     if (!Recurse)
-	return FALSE;
+	return false;
 
     /* Try to make each of comp and comp/foo in turn. */
     for (p = (Name[0] == '/') ? &Name[1] : Name; *p; p++)
@@ -49,7 +48,7 @@ BOOL MakeDirectory(char *Name, BOOL Recurse)
 	    made = MakeDir(Name);
 	    *p = '/';
 	    if (!made)
-		return FALSE;
+		return false;
 	}
 
     return MakeDir(Name);
