@@ -1,4 +1,4 @@
-/*  $Id: vector.c,v 1.5.2.1 2004/04/07 06:47:44 rra Exp $
+/*  $Id: vector.c 8962 2010-02-08 20:54:57Z iulius $
 **
 **  Vector handling (counted lists of char *'s).
 **
@@ -27,7 +27,7 @@
 #include <ctype.h>
 
 #include "inn/vector.h"
-#include "libinn.h"
+#include "inn/libinn.h"
 
 /*
 **  Allocate a new, empty vector.
@@ -296,7 +296,7 @@ split_space_count(const char *string)
 
 /*
 **  Given a string, split it at whitespace to form a vector, copying each
-**  string segment.  If the fourth argument isn't NULL, reuse that vector;
+**  string segment.  If the second argument isn't NULL, reuse that vector;
 **  otherwise, allocate a new one.  Any number of consecutive whitespace
 **  characters is considered a single separator.
 */
@@ -328,7 +328,7 @@ vector_split_space(const char *string, struct vector *vector)
 
 /*
 **  Given a string, split it at whitespace to form a vector, destructively
-**  modifying the string to nul-terminate each segment.  If the fourth
+**  modifying the string to nul-terminate each segment.  If the second
 **  argument isn't NULL, reuse that vector; otherwise, allocate a new one.
 **  Any number of consecutive whitespace characters is considered a single
 **  separator.
@@ -407,3 +407,27 @@ cvector_join(const struct cvector *vector, const char *seperator)
 
     return string;
 }
+
+
+/*
+**  Given a vector and a path to a program, exec that program with the vector
+**  as its arguments.  This requires adding a NULL terminator to the vector
+**  and casting it appropriately.
+*/
+int
+vector_exec(const char *path, struct vector *vector)
+{
+    if (vector->allocated == vector->count)
+        vector_resize(vector, vector->count + 1);
+    vector->strings[vector->count] = NULL;
+    return execv(path, (char * const *) vector->strings);
+}
+
+int
+cvector_exec(const char *path, struct cvector *vector)
+{
+    if (vector->allocated == vector->count)
+        cvector_resize(vector, vector->count + 1);
+    vector->strings[vector->count] = NULL;
+    return execv(path, (char * const *) vector->strings);
+}
