@@ -1,4 +1,4 @@
-/*  $Id: misc.c 8789 2009-11-15 09:27:31Z iulius $
+/*  $Id: misc.c 9259 2011-07-17 18:22:50Z iulius $
 **
 **  Miscellaneous support routines.
 */
@@ -518,5 +518,8 @@ CMDstarttls(int ac UNUSED, char *av[] UNUSED)
         GRPcount = 0;
         PERMgroupmadeinvalid = false;
     }
+
+    /* Reset our read buffer so as to prevent plaintext command injection. */
+    line_reset(&NNTPline);
 }
 #endif /* HAVE_SSL */
