@@ -1,4 +1,4 @@
-/*  $Id: perm.c 9024 2010-03-21 16:49:30Z iulius $
+/*  $Id: perm.c 9317 2011-08-06 20:24:27Z iulius $
 **
 **  How to figure out where a user comes from, and what that user can do once
 **  we know who sie is.
@@ -625,7 +625,10 @@ method_parse(METHOD *method, CONFFILE *f, CONFTOKEN *tok, int auth)
 static void
 authdecl_parse(AUTHGROUP *curauth, CONFFILE *f, CONFTOKEN *tok)
 {
-    int oldtype,boolval;
+    int oldtype;
+#ifdef HAVE_SSL
+    int boolval;
+#endif
     METHOD *m;
     bool bit;
     char buff[SMBUF], *oldname, *p;
@@ -645,6 +648,7 @@ authdecl_parse(AUTHGROUP *curauth, CONFFILE *f, CONFTOKEN *tok)
 	ReportError(f, buff);
     }
 
+#ifdef HAVE_SSL
     if (strcasecmp(tok->name, "on") == 0
         || strcasecmp(tok->name, "true") == 0
         || strcasecmp(tok->name, "yes") == 0)
@@ -655,6 +659,7 @@ authdecl_parse(AUTHGROUP *curauth, CONFFILE *f, CONFTOKEN *tok)
 	boolval = false;
     else
 	boolval = -1;
+#endif
 
     switch (oldtype) {
       case PERMkey:
@@ -1064,6 +1069,7 @@ PERMvectortoaccess(ACCESSGROUP *acc, const char *name,
     SetDefaultAccess(acc);
     str = xstrdup(name);
     acc->name = str;
+    file->filename = str;
 
     for (i = 0; i <= access_vec->count; i++) {
       tok = CONFgettoken(PERMtoks, file);
@@ -1072,6 +1078,9 @@ PERMvectortoaccess(ACCESSGROUP *acc, const char *name,
         accessdecl_parse(acc, file, tok);
       }
     }
+
+    /* No need to free str here; it is used by acc, and will be
+     * free'd when the access block is free'd. */
     free(file);
     return;
 }
@@ -1603,7 +1612,7 @@ PERMgetpermissions(void)
 
         access_realms[0] = xcalloc(1, sizeof(ACCESSGROUP));
 
-        PERMvectortoaccess(access_realms[0], "perl-dynamic", access_vec);
+        PERMvectortoaccess(access_realms[0], "perl_access-block", access_vec);
 
         vector_free(access_vec);
       } else {
@@ -1638,7 +1647,7 @@ PERMgetpermissions(void)
             access_realms[0] = xcalloc(1, sizeof(ACCESSGROUP));
             memset(access_realms[0], 0, sizeof(ACCESSGROUP));
 
-            PERMvectortoaccess(access_realms[0], "python-dynamic", access_vec);
+            PERMvectortoaccess(access_realms[0], "python_access-block", access_vec);
 
             vector_free(access_vec);
         } else {
