@@ -1,4 +1,4 @@
-/*  $Id: cnfs.c 9024 2010-03-21 16:49:30Z iulius $
+/*  $Id: cnfs.c 9320 2011-08-06 20:43:03Z iulius $
 **
 **  Storage manager module for Cyclic News File System method.
 */
@@ -1741,7 +1741,7 @@ cnfs_next(ARTHANDLE *article, const RETRTYPE amount)
 	}
     }
     if (innconf->cnfscheckfudgesize != 0 && innconf->maxartsize != 0 &&
-	(ntohl(cah.size) > innconf->maxartsize + innconf->cnfscheckfudgesize)) {
+	((unsigned int) ntohl(cah.size) > innconf->maxartsize + innconf->cnfscheckfudgesize)) {
 	art->data = NULL;
 	art->len = 0;
 	art->token = NULL;
