@@ -1,11 +1,11 @@
-/*  $Id: expire.c 8409 2009-04-11 22:13:41Z iulius $
+/*  $Id: expire.c 9024 2010-03-21 16:49:30Z iulius $
 **
 **  Code for overview-driven expiration.
 **
 **  In order to expire on a per-newsgroup (instead of per-storage-class)
 **  basis, one has to use overview-driven expiration.  This contains all of
-**  the code to do that.  It provides OVgroupbasedexpire, OVhisthasmsgid, and
-**  OVgroupmatch for the use of various overview methods.
+**  the code to do that.  It provides OVgroupbasedexpire and OVhisthasmsgid
+**  for the use of various overview methods.
 */
 
 #include "config.h"
@@ -101,8 +101,6 @@ bool     OVusepost;
 bool     OVkeep;
 bool     OVearliest;
 bool     OVquiet;
-int      OVnumpatterns;
-char **  OVpatterns;
 
 
 /*
@@ -286,7 +284,7 @@ EXPgetnum(int line, char *word, time_t *v, const char *name)
                 break;
             SawDot = true;
         }
-        else if (!CTYPE(isdigit, (int)*p))
+        else if (!isdigit((unsigned char) *p))
             break;
     if (*p) {
         fprintf(stderr, "Line %d, bad `%c' character in %s field\n",
@@ -360,7 +358,7 @@ EXPreadfile(FILE *F)
         else
             p = buff + strlen(buff);
         while (--p >= buff) {
-            if (isspace((int)*p))
+            if (isspace((unsigned char) *p))
                 *p = '\0';
             else
                 break;
@@ -712,7 +710,7 @@ OVgroupbasedexpire(TOKEN token, const char *group, const char *data,
             EXPoverindexdrop++;
             return true;
         }
-        when = parsedate_rfc2822_lax(p);
+        when = parsedate_rfc5322_lax(p);
         if (when == (time_t) -1) {
             EXPoverindexdrop++;
             return true;
@@ -813,32 +811,6 @@ OVhisthasmsgid(struct history *h, const char *data)
     return HISlookup(h, p, NULL, NULL, NULL, NULL);
 }
 
-bool
-OVgroupmatch(const char *group)
-{
-    int i;
-    bool wanted = false;
-
-    if (OVnumpatterns == 0 || group == NULL)
-        return true;
-    for (i = 0; i < OVnumpatterns; i++) {
-        switch (OVpatterns[i][0]) {
-        case '!':
-            if (!wanted && uwildmat(group, &OVpatterns[i][1]))
-                break;
-        case '@':
-            if (uwildmat(group, &OVpatterns[i][1])) {
-                return false;
-            }
-            break;
-        default:
-            if (uwildmat(group, OVpatterns[i]))
-                wanted = true;
-        }
-    }
-    return wanted;
-}
-
 void
 OVEXPcleanup(void)
 {
@@ -855,11 +827,6 @@ OVEXPcleanup(void)
         }
         EXPprocessed = EXPunlinked = EXPoverindexdrop = 0;
     }
-    if (innconf->ovgrouppat != NULL) {
-        for (i = 0 ; i < OVnumpatterns ; i++)
-            free(OVpatterns[i]);
-        free(OVpatterns);
-    }
     for (bg = EXPbadgroups; bg; bg = bgnext) {
         bgnext = bg->Next;
         free(bg->Name);
