@@ -1,4 +1,4 @@
-/*  $Id: tdx-cache.c 5394 2002-04-04 22:55:24Z rra $
+/*  $Id: tdx-cache.c 8999 2010-03-14 20:59:54Z iulius $
 **
 **  Cache functions for open overview data files.
 **
@@ -14,8 +14,8 @@
 
 #include "inn/hashtab.h"
 #include "inn/messages.h"
-#include "libinn.h"
-#include "storage.h"
+#include "inn/libinn.h"
+#include "inn/storage.h"
 #include "tdx-private.h"
 
 /* Returned to callers as an opaque data type, this struct holds all of the
@@ -197,9 +197,12 @@ tdx_cache_insert(struct cache *cache, HASH hash, struct group_data *data)
 void
 tdx_cache_delete(struct cache *cache, HASH hash)
 {
-    if (!hash_delete(cache->hashtable, &hash))
+    if (!hash_delete(cache->hashtable, &hash)) {
         warn("tradindexed: unable to remove cache entry for %s",
              HashToText(hash));
+    } else {
+        cache->count--;
+    }
 }
 
 
