@@ -1,4 +1,4 @@
-/*  $Id: tdx-group.c,v 1.22.2.4 2004/01/05 21:40:07 rra Exp $
+/*  $Id: tdx-group.c 8476 2009-05-18 03:59:10Z eagle $
 **
 **  Group index handling for the tradindexed overview method.
 **
@@ -96,8 +96,8 @@
 #include "inn/mmap.h"
 #include "inn/qio.h"
 #include "inn/vector.h"
-#include "libinn.h"
-#include "paths.h"
+#include "inn/libinn.h"
+#include "inn/paths.h"
 #include "tdx-private.h"
 #include "tdx-structure.h"
 
@@ -380,7 +380,7 @@ index_expand(struct group_index *index)
         index->header->freelist.recno = i;
     }
 
-    mapcntl(index->header, index_file_size(index->count), MS_ASYNC);
+    inn_msync_page(index->header, index_file_size(index->count), MS_ASYNC);
     return true;
 }
 
@@ -459,7 +459,7 @@ entry_splice(struct group_entry *entry, int *parent)
 {
     *parent = entry->next.recno;
     entry->next.recno = -1;
-    mapcntl(parent, sizeof(*parent), MS_ASYNC);
+    inn_msync_page(parent, sizeof(*parent), MS_ASYNC);
 }
 
 
@@ -480,8 +480,8 @@ index_add(struct group_index *index, struct group_entry *entry)
     }
     entry->next.recno = index->header->hash[bucket].recno;
     index->header->hash[bucket].recno = entry_loc(index, entry);
-    mapcntl(&index->header->hash[bucket], sizeof(struct loc), MS_ASYNC);
-    mapcntl(entry, sizeof(*entry), MS_ASYNC);
+    inn_msync_page(&index->header->hash[bucket], sizeof(struct loc), MS_ASYNC);
+    inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
 }
 
 
@@ -529,8 +529,8 @@ freelist_add(struct group_index *index, struct group_entry *entry)
 {
     entry->next.recno = index->header->freelist.recno;
     index->header->freelist.recno = entry_loc(index, entry);
-    mapcntl(&index->header->freelist, sizeof(struct loc), MS_ASYNC);
-    mapcntl(entry, sizeof(*entry), MS_ASYNC);
+    inn_msync_page(&index->header->freelist, sizeof(struct loc), MS_ASYNC);
+    inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
 }
 
 
@@ -600,7 +600,7 @@ tdx_index_entry(struct group_index *index, const char *group)
         return NULL;
     entry = index->entries + loc;
     if (innconf->tradindexedmmap && innconf->nfsreader)
-	mapcntl(entry, sizeof *entry, MS_INVALIDATE);
+	inn_msync_page(entry, sizeof *entry, MS_INVALIDATE);
     return entry;
 }
 
@@ -630,7 +630,7 @@ tdx_index_add(struct group_index *index, const char *group, ARTNUM low,
         entry = &index->entries[loc];
         if (entry->flag != *flag) {
             entry->flag = *flag;
-            mapcntl(entry, sizeof(*entry), MS_ASYNC);
+            inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
         }
         return true;
     }
@@ -645,7 +645,7 @@ tdx_index_add(struct group_index *index, const char *group, ARTNUM low,
         }
     loc = index->header->freelist.recno;
     index->header->freelist.recno = index->entries[loc].next.recno;
-    mapcntl(&index->header->freelist, sizeof(struct loc), MS_ASYNC);
+    inn_msync_page(&index->header->freelist, sizeof(struct loc), MS_ASYNC);
 
     /* Initialize the entry. */
     entry = &index->entries[loc];
@@ -820,11 +820,11 @@ tdx_data_add(struct group_index *index, struct group_entry *entry,
         old_base = entry->base;
         entry->indexinode = data->indexinode;
         entry->base = data->base;
-        mapcntl(entry, sizeof(*entry), MS_ASYNC);
+        inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
         if (!tdx_data_pack_finish(data)) {
             entry->base = old_base;
             entry->indexinode = old_inode;
-            mapcntl(entry, sizeof(*entry), MS_ASYNC);
+            inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
             goto fail;
         }
     }
@@ -839,7 +839,14 @@ tdx_data_add(struct group_index *index, struct group_entry *entry,
     if (entry->high < article->number)
         entry->high = article->number;
     entry->count++;
-    mapcntl(entry, sizeof(*entry), MS_ASYNC);
+
+    /* Used to know that we have to remap the data file owing to our
+       OVSTATICSEARCH (an article whose number is lower than the highest has
+       been added at the end of the file). */
+    if (data->high > article->number)
+        data->remapoutoforder = true;
+
+    inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
     index_lock_group(index->fd, offset, INN_LOCK_UNLOCK);
     return true;
 
@@ -880,7 +887,7 @@ tdx_index_rebuild_finish(struct group_index *index, struct group_entry *entry,
     *entry = *new;
     entry->indexinode = new_inode;
     new->indexinode = new_inode;
-    mapcntl(entry, sizeof(*entry), MS_ASYNC);
+    inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
     offset = entry - index->entries;
     index_lock_group(index->fd, offset, INN_LOCK_UNLOCK);
     return true;
@@ -930,12 +937,12 @@ tdx_expire(const char *group, ARTNUM *low, struct history *history)
     old_base = entry->base;
     entry->indexinode = new_entry.indexinode;
     entry->base = new_entry.base;
-    mapcntl(entry, sizeof(*entry), MS_ASYNC);
+    inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
     tdx_data_close(data);
     if (!tdx_data_rebuild_finish(group)) {
         entry->base = old_base;
         entry->indexinode = old_inode;
-        mapcntl(entry, sizeof(*entry), MS_ASYNC);
+        inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
         goto fail;
     }
 
@@ -1043,7 +1050,7 @@ hashmap_load(void)
     struct hashmap *group;
     HASH grouphash;
 
-    activepath = concatpath(innconf->pathdb, _PATH_ACTIVE);
+    activepath = concatpath(innconf->pathdb, INN_PATH_ACTIVE);
     active = QIOopen(activepath);
     free(activepath);
     if (active == NULL)
@@ -1070,7 +1077,8 @@ hashmap_load(void)
         hash_insert(hash, &group->hash, group);
         line = QIOread(active);
     }
-    cvector_free(data);
+    if (data != NULL)
+        cvector_free(data);
     QIOclose(active);
     return hash;
 }
@@ -1169,7 +1177,7 @@ index_audit_loc(struct group_index *index, int *loc, long number,
 
     if (fix && error) {
         *loc = -1;
-        mapcntl(loc, sizeof(*loc), MS_ASYNC);
+        inn_msync_page(loc, sizeof(*loc), MS_ASYNC);
     }
 }
 
@@ -1187,7 +1195,7 @@ index_audit_deleted(struct group_entry *entry, long number, bool fix)
              number);
         if (fix) {
             HashClear(&entry->hash);
-            mapcntl(entry, sizeof(*entry), MS_ASYNC);
+            inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
         }
     }
 }
@@ -1327,7 +1335,7 @@ index_audit_group(struct group_index *index, struct group_entry *entry,
     } else {
         if (entry->flag != group->flag) {
             entry->flag = group->flag;
-            mapcntl(entry, sizeof(*entry), MS_ASYNC);
+            inn_msync_page(entry, sizeof(*entry), MS_ASYNC);
         }
         tdx_data_audit(group->name, entry, fix);
     }
@@ -1410,6 +1418,10 @@ tdx_index_audit(bool fix)
        need to map hashes to group names, so load a hash of the active file to
        do that resolution. */
     hashmap = hashmap_load();
+    if (hashmap == NULL) {
+        warn("tradindexed: cannot hash active file");
+        return;
+    }
     data.index = index;
     data.fix = fix;
     hash_traverse(hashmap, index_audit_active, &data);
@@ -1419,6 +1431,5 @@ tdx_index_audit(bool fix)
             continue;
         index_audit_group(index, entry, hashmap, fix);
     }
-    if (hashmap != NULL)
-        hash_free(hashmap);
+    hash_free(hashmap);
 }
