@@ -1,4 +1,4 @@
-/*  $Id: tdx-util.c 8409 2009-04-11 22:13:41Z iulius $
+/*  $Id: tdx-util.c 9024 2010-03-21 16:49:30Z iulius $
 **
 **  Utility for managing a tradindexed overview spool.
 **
@@ -171,7 +171,7 @@ check_number(const char *string)
     const char *p;
 
     for (p = string; *p != '\0'; p++)
-        if (!CTYPE(isdigit, *p))
+        if (!isdigit((unsigned char) *p))
             return false;
     return true;
 }
@@ -295,7 +295,7 @@ group_rebuild(const char *group, const char *path)
         die("cannot open group index");
     entry = tdx_index_entry(index, group);
     if (entry == NULL) {
-        if (!tdx_index_add(index, group, 1, 0, "y"))
+        if (!tdx_index_add(index, group, 1, 0, NF_FLAG_OK_STRING))
             die("cannot create group %s", group);
         entry = tdx_index_entry(index, group);
         if (entry == NULL)
