@@ -1,6 +1,6 @@
-/*  $Id: trash.h 4267 2001-01-04 06:02:02Z rra $
+/*  $Id: trash.h 8818 2009-11-17 18:58:24Z iulius $
 **
-**  trashing articles method header
+**  Storage manager module header for trash method.
 */
 
 #ifndef __TRASH_H__
@@ -12,12 +12,13 @@
 bool trash_init(SMATTRIBUTE *attr);
 TOKEN trash_store(const ARTHANDLE article, const STORAGECLASS class);
 ARTHANDLE *trash_retrieve(const TOKEN token, const RETRTYPE amount);
-ARTHANDLE *trash_next(const ARTHANDLE *article, const RETRTYPE amount);
+ARTHANDLE *trash_next(ARTHANDLE *article, const RETRTYPE amount);
 void trash_freearticle(ARTHANDLE *article);
 bool trash_cancel(TOKEN token);
 bool trash_ctl(PROBETYPE type, TOKEN *token, void *value);
 bool trash_flushcacheddata(FLUSHTYPE type);
 void trash_printfiles(FILE *file, TOKEN token, char **xref, int ngroups);
+char *trash_explaintoken(const TOKEN token);
 void trash_shutdown(void);
 
 #endif
