@@ -1,4 +1,4 @@
-/* $Id: innconf-t.c 6084 2002-12-27 07:24:55Z rra $ */
+/* $Id: innconf-t.c 9280 2011-07-17 19:36:24Z iulius $ */
 /* innconf test suite. */
 
 #include "config.h"
@@ -8,9 +8,16 @@
 #include "inn/messages.h"
 #include "libtest.h"
 
+/* We will have strings, integers, bools and lists. */
 static const char grep[] =
-"egrep 'mta|organization|ovmethod|hismethod|path|pgpverify'\
- ../../samples/inn.conf > config/tmp";
+"(egrep '^(mta|organization|ovmethod|hismethod|path|port|extraoverview|pgpverify)'\
+ ../../samples/inn.conf; echo 'domain: \"news.example.org\"';\
+ echo 'fromhost: \"news.example.org\"';) > config/tmp";
+
+static const char cat[] =
+"(cat ../../samples/inn.conf | egrep -v '^#(domain|fromhost)';\
+ echo 'domain: \"news.example.org\"'; echo 'fromhost: \"news.example.org\"';)\
+ > config/tmp2";
 
 int
 main(void)
@@ -18,19 +25,25 @@ main(void)
     struct innconf *standard;
     FILE *config;
 
-    if (access("config/valid", F_OK) < 0)
-        if (access("lib/config/valid", F_OK) == 0)
-            chdir("lib");
+    if (access("../data/config/valid", F_OK) == 0)
+        chdir("../data");
+    else if (access("data/config/valid", F_OK) == 0)
+        chdir("data");
+    else if (access("tests/data/config/valid", F_OK) == 0)
+        chdir("tests/data");
 
-    puts("9");
+    test_init(9);
 
-    ok(1, innconf_read("../../samples/inn.conf"));
+    if (system(cat) != 0)
+        die("Unable to create stripped configuration file");
+    ok(1, innconf_read("config/tmp2"));
     standard = innconf;
     innconf = NULL;
     if (system(grep) != 0)
         die("Unable to create stripped configuration file");
     ok(2, innconf_read("config/tmp"));
     unlink("config/tmp");
+    unlink("config/tmp2");
     ok(3, innconf_compare(standard, innconf));
     innconf_free(standard);
     innconf_free(innconf);
@@ -52,7 +65,7 @@ main(void)
     fclose(config);
     ok(7, !innconf_check("config/tmp"));
     unlink("config/tmp");
-    ok_string(8, "config/tmp:25: unknown parameter foo\n", errors);
+    ok_string(8, "config/tmp:28: unknown parameter foo\n", errors);
     errors_uncapture();
     free(errors);
     errors = NULL;
