@@ -1,4 +1,4 @@
-/* $Id: messages-t.c 5638 2002-08-23 22:52:53Z rra $ */
+/* $Id: messages-t.c 9280 2011-07-17 19:36:24Z iulius $ */
 /* Test suite for error handling routines. */
 
 #include "config.h"
@@ -9,7 +9,8 @@
 #include <sys/wait.h>
 
 #include "inn/messages.h"
-#include "libinn.h"
+#include "inn/libinn.h"
+#include "libtest.h"
 
 #define END     (char *) 0
 
@@ -25,6 +26,7 @@ run_test(test_function_t function, char *buf, size_t buflen)
     int fds[2];
     pid_t child;
     ssize_t count, status;
+    int rval;
 
     /* Flush stdout before we start to avoid odd forking issues. */
     fflush(stdout);
@@ -58,10 +60,10 @@ run_test(test_function_t function, char *buf, size_t buflen)
                 count += status;
         } while (status > 0);
         buf[count < 0 ? 0 : count] = '\0';
-        if (waitpid(child, &status, 0) == (pid_t) -1)
+        if (waitpid(child, &rval, 0) == (pid_t) -1)
             sysdie("waitpid failed");
     }
-    return status;
+    return rval;
 }
 
 /* Test functions. */
@@ -108,33 +110,33 @@ static void test11(void) {
     sysdie("fatal");
 }
 
-static void log(int len, const char *format, va_list args, int error) {
+static void log_msg(int len, const char *format, va_list args, int error) {
     fprintf(stderr, "%d %d ", len, error);
     vfprintf(stderr, format, args);
     fprintf(stderr, "\n");
 }
 
 static void test12(void) {
-    message_handlers_warn(1, log);
+    message_handlers_warn(1, log_msg);
     warn("warning");
 }
 static void test13(void) {
-    message_handlers_die(1, log);
+    message_handlers_die(1, log_msg);
     die("fatal");
 }
 static void test14(void) {
-    message_handlers_warn(2, log, log);
+    message_handlers_warn(2, log_msg, log_msg);
     errno = EPERM;
     syswarn("warning");
 }
 static void test15(void) {
-    message_handlers_die(2, log, log);
+    message_handlers_die(2, log_msg, log_msg);
     message_fatal_cleanup = return10;
     errno = EPERM;
     sysdie("fatal");
 }
 static void test16(void) {
-    message_handlers_warn(2, message_log_stderr, log);
+    message_handlers_warn(2, message_log_stderr, log_msg);
     message_program_name = "test16";
     errno = EPERM;
     syswarn("warning");
@@ -144,32 +146,25 @@ static void test18(void) {
     message_program_name = "test18";
     notice("notice");
 }
-static void test19(void) { trace(TRACE_PROGRAM, "tracing"); }
-static void test20(void) { debug("debug"); }
-static void test21(void) {
-    message_handlers_notice(1, log);
+static void test19(void) { debug("debug"); }
+static void test20(void) {
+    message_handlers_notice(1, log_msg);
     notice("foo");
 }
-static void test22(void) {
-    message_handlers_trace(1, log);
-    message_trace_enable(TRACE_PROGRAM, true);
-    trace(TRACE_PROGRAM, "foo");
-    trace(TRACE_NETWORK, "bar");
-}
-static void test23(void) {
+static void test21(void) {
     message_handlers_debug(1, message_log_stdout);
     message_program_name = "test23";
     debug("baz");
 }
-static void test24(void) {
+static void test22(void) {
     message_handlers_die(0);
     die("hi mom!");
 }
-static void test25(void) {
+static void test23(void) {
     message_handlers_warn(0);
     warn("this is a test");
 }
-static void test26(void) {
+static void test24(void) {
     notice("first");
     message_handlers_notice(0);
     notice("second");
@@ -188,15 +183,15 @@ test_error(int n, int status, const char *output, test_function_t function)
 
     real_status = run_test(function, buf, sizeof(buf));
     if (!WIFEXITED(real_status) || status != WEXITSTATUS(real_status)) {
-        printf("  unexpected exit status %d\n", real_status);
+        diag("  unexpected exit status %d\n", real_status);
         succeeded = 0;
     }
     if (strcmp(output, buf)) {
-        printf("  unexpected output: %s", buf);
-        printf("    expected output: %s", output);
+        diag("  unexpected output: %s", buf);
+        diag("    expected output: %s", output);
         succeeded = 0;
     }
-    printf("%sok %d\n", succeeded ? "" : "not ", n);
+    ok(n, succeeded);
 }
 
 /* Given the test number, intended status, intended message sans the
@@ -219,7 +214,7 @@ main(void)
 {
     char buff[32];
 
-    puts("26");
+    test_init(24);
 
     test_error(1, 0, "warning\n", test1);
     test_error(2, 1, "fatal\n", test2);
@@ -251,15 +246,13 @@ main(void)
     test_error(17, 0, "notice\n", test17);
     test_error(18, 0, "test18: notice\n", test18);
     test_error(19, 0, "", test19);
-    test_error(20, 0, "", test20);
-    test_error(21, 0, "3 0 foo\n", test21);
-    test_error(22, 0, "3 0 foo\n", test22);
-    test_error(23, 0, "test23: baz\n", test23);
+    test_error(20, 0, "3 0 foo\n", test20);
+    test_error(21, 0, "test23: baz\n", test21);
 
     /* Make sure that it's possible to turn off a message type entirely. */ 
-    test_error(24, 1, "", test24);
-    test_error(25, 0, "", test25);
-    test_error(26, 0, "first\nthird\n", test26);
+    test_error(22, 1, "", test22);
+    test_error(23, 0, "", test23);
+    test_error(24, 0, "first\nthird\n", test24);
 
     return 0;
 }
