@@ -1,4 +1,4 @@
-/* $Id: api-t.c 8468 2009-05-16 12:28:24Z iulius $ */
+/* $Id: api-t.c 9280 2011-07-17 19:36:24Z iulius $ */
 /* Test suite for overview API. */
 
 #include "config.h"
@@ -158,7 +158,7 @@ overview_load(const char *data, struct overview *overview)
     char buffer[4096];
     char *start;
     unsigned long artnum;
-    struct overview_group stats = { 0, 0, 0, 'y' };
+    struct overview_group stats = { 0, 0, 0, NF_FLAG_OK };
     struct overview_data article;
 
     /* Run through the overview data.  Each time we see a group, we update our
@@ -251,8 +251,9 @@ overview_verify_groups(void *data, void *cookie)
              stats.count, group->count);
         verify->status = false;
     }
-    if (stats.flag != 'y') {
-        warn("Flag wrong for %s: %c != y", group->group, stats.flag);
+    if (stats.flag != NF_FLAG_OK) {
+        warn("Flag wrong for %s: %c != %c", group->group, stats.flag,
+             NF_FLAG_OK);
         verify->status = false;
     }
 }
@@ -263,7 +264,7 @@ check_data(const char *group, ARTNUM artnum, const char *expected,
            struct overview_data *data)
 {
     bool status = true;
-    time_t expires;
+    time_t expires UNUSED; // See below why it is still unused.
     char *saw;
 
     if (artnum != data->number) {
@@ -694,15 +695,15 @@ main(void)
     fake_innconf();
     innconf->ovmethod = xstrdup("tradindexed");
     innconf->tradindexedmmap = true;
-    printf("# tradindexed with mmap\n");
+    diag("tradindexed with mmap");
     n = overview_tests(1);
 
-    printf("# tradindexed without mmap\n");
+    diag("tradindexed without mmap");
     n = overview_mmap_tests(n);
 
     free(innconf->ovmethod);
     innconf->ovmethod = xstrdup("buffindexed");
-    printf("# buffindexed\n");
+    diag("buffindexed");
     n = overview_tests(n);
 
     return 0;
